@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 4.5 2017/06/10 15:08:56 kls Exp $
+ * $Id: channels.c 5.3 2024/03/02 16:21:16 kls Exp $
  */
 
 #include "channels.h"
@@ -97,23 +97,32 @@ cChannel& cChannel::operator= (const cChannel &Channel)
   provider = strcpyrealloc(provider, Channel.provider);
   portalName = strcpyrealloc(portalName, Channel.portalName);
   memcpy(&__BeginData__, &Channel.__BeginData__, (char *)&Channel.__EndData__ - (char *)&Channel.__BeginData__);
-  nameSource = NULL; // these will be recalculated automatically
-  nameSourceMode = 0;
-  shortNameSource = NULL;
+  UpdateNameSource();
   parameters = Channel.parameters;
   return *this;
 }
 
+void cChannel::UpdateNameSource(void)
+{
+  if (Setup.ShowChannelNamesWithSource == 0) {
+     nameSource = NULL;
+     shortNameSource = NULL;
+     return;
+     }
+
+  if (Setup.ShowChannelNamesWithSource == 1)
+     nameSource = cString::sprintf("%s (%c)", name, cSource::ToChar(source));
+  else
+     nameSource = cString::sprintf("%s (%s)", name, *cSource::ToString(source));
+
+  shortNameSource = cString::sprintf("%s (%c)", shortName, cSource::ToChar(source));
+}
+
 const char *cChannel::Name(void) const
 {
   if (Setup.ShowChannelNamesWithSource && !groupSep) {
-     if (isempty(nameSource) || nameSourceMode != Setup.ShowChannelNamesWithSource) {
-        if (Setup.ShowChannelNamesWithSource == 1)
-           nameSource = cString::sprintf("%s (%c)", name, cSource::ToChar(source));
-        else
-           nameSource = cString::sprintf("%s (%s)", name, *cSource::ToString(source));
-        }
-     return nameSource;
+     if (!isempty(nameSource))
+        return nameSource;
      }
   return name;
 }
@@ -123,9 +132,8 @@ const char *cChannel::ShortName(bool OrName) const
   if (OrName && isempty(shortName))
      return Name();
   if (Setup.ShowChannelNamesWithSource && !groupSep) {
-     if (isempty(shortNameSource))
-        shortNameSource = cString::sprintf("%s (%c)", shortName, cSource::ToChar(source));
-     return shortNameSource;
+     if (!isempty(shortNameSource))
+        return shortNameSource;
      }
   return shortName;
 }
@@ -145,15 +153,18 @@ int cChannel::Transponder(int Frequency, char Polarization)
 
 int cChannel::Transponder(void) const
 {
-  int tf = frequency;
-  while (tf > 20000)
-        tf /= 1000;
-  if (IsSat()) {
-     const char *p = strpbrk(parameters, "HVLRhvlr"); // lowercase for backwards compatibility
-     if (p)
-        tf = Transponder(tf, *p);
+  if (!transponder) {
+     int tf = frequency;
+     while (tf > 20000)
+           tf /= 1000;
+     if (IsSat()) {
+        const char *p = strpbrk(parameters, "HVLRhvlr"); // lowercase for backwards compatibility
+        if (p)
+           tf = Transponder(tf, *p);
+        }
+     transponder = tf;
      }
-  return tf;
+  return transponder;
 }
 
 int cChannel::Modification(int Mask) const
@@ -167,6 +178,7 @@ void cChannel::CopyTransponderData(const cChannel *Channel)
 {
   if (Channel) {
      frequency    = Channel->frequency;
+     transponder  = Channel->transponder;
      source       = Channel->source;
      srate        = Channel->srate;
      parameters   = Channel->parameters;
@@ -195,12 +207,11 @@ bool cChannel::SetTransponderData(int Source, int Frequency, int Srate, const ch
      cString OldTransponderData = TransponderDataToString();
      source = Source;
      frequency = Frequency;
+     transponder = 0;
      srate = Srate;
      parameters = Parameters;
      schedule = NULL;
-     nameSource = NULL;
-     nameSourceMode = 0;
-     shortNameSource = NULL;
+     UpdateNameSource();
      if (Number() && !Quiet) {
         dsyslog("changing transponder data of channel %d (%s) from %s to %s", Number(), name, *OldTransponderData, *TransponderDataToString());
         modification |= CHANNELMOD_TRANSP;
@@ -265,15 +276,12 @@ bool cChannel::SetName(const char *Name, const char *ShortName, const char *Prov
            dsyslog("changing name of channel %d from '%s,%s;%s' to '%s,%s;%s'", Number(), name, shortName, provider, Name, ShortName, Provider);
            modification |= CHANNELMOD_NAME;
            }
-        if (nn) {
+        if (nn)
            name = strcpyrealloc(name, Name);
-           nameSource = NULL;
-           nameSourceMode = 0;
-           }
-        if (ns) {
+        if (ns)
            shortName = strcpyrealloc(shortName, ShortName);
-           shortNameSource = NULL;
-           }
+        if (nn || ns)
+           UpdateNameSource();
         if (np)
            provider = strcpyrealloc(provider, Provider);
         return true;
@@ -655,6 +663,7 @@ bool cChannel::Parse(const char *s)
         if (parambuf && sourcebuf && vpidbuf && apidbuf) {
            parameters = parambuf;
            ok = (source = cSource::FromString(sourcebuf)) >= 0;
+           transponder = 0;
 
            char *p;
            if ((p = strchr(vpidbuf, '=')) != NULL) {
@@ -798,9 +807,7 @@ bool cChannel::Parse(const char *s)
         free(tpidbuf);
         free(caidbuf);
         free(namebuf);
-        nameSource = NULL;
-        nameSourceMode = 0;
-        shortNameSource = NULL;
+        UpdateNameSource();
         if (!GetChannelID().Valid()) {
            esyslog("ERROR: channel data results in invalid ID!");
            return false;
@@ -946,6 +953,25 @@ void cChannels::ReNumber(void)
       }
 }
 
+bool cChannels::MoveNeedsDecrement(cChannel *From, cChannel *To)
+{
+  int Number = From->Number();
+  if (Number < To->Number()) {
+     for (cChannel *Channel = Next(From); Channel; Channel = Next(Channel)) {
+         if (Channel == To)
+            break;
+         if (Channel->GroupSep()) {
+            if (Channel->Number() > Number)
+               Number = Channel->Number();
+            }
+         else
+            Number++;
+         }
+     return Number == To->Number();
+     }
+  return false;
+}
+
 void cChannels::Del(cChannel *Channel)
 {
   UnhashChannel(Channel);
