@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 5.8 2021/12/27 13:31:29 kls Exp $
+ * $Id: config.h 5.18 2024/07/15 09:07:09 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,13 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "2.6.0"
-#define VDRVERSNUM   20600  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "2.6.9"
+#define VDRVERSNUM   20609  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "2.6.0"
-#define APIVERSNUM   20600  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "2.6.9"
+#define APIVERSNUM   20609  // Version * 10000 + Major * 100 + Minor
 
 // When loading plugins, VDR searches them by their APIVERSION, which
 // may be smaller than VDRVERSION in case there have been no changes to
@@ -291,6 +291,8 @@ public:
   int SubtitleOffset;
   int SubtitleFgTransparency, SubtitleBgTransparency;
   int EPGLanguages[I18N_MAX_LANGUAGES + 1];
+  int EPGScanMaxChannel;
+  int EPGPauseAfterScan;
   int EPGScanTimeout;
   int EPGBugfixLevel;
   int EPGLinger;
