@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 5.3 2021/06/08 14:57:26 kls Exp $
+ * $Id: device.c 5.14 2024/07/06 11:19:21 kls Exp $
  */
 
 #include "device.h"
@@ -95,7 +95,9 @@ cDevice::cDevice(void)
 
   camSlot = NULL;
 
+  occupiedFrom = 0;
   occupiedTimeout = 0;
+  occupiedPriority = MINPRIORITY;
 
   player = NULL;
   isPlayingVideo = false;
@@ -249,7 +251,7 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
 {
   // Collect the current priorities of all CAM slots that can decrypt the channel:
   int NumCamSlots = CamSlots.Count();
-  int SlotPriority[NumCamSlots];
+  int SlotPriority[NumCamSlots + 1]; // +1 to avoid a zero sized array in case there are no CAM slots
   int NumUsableSlots = 0;
   bool InternalCamNeeded = false;
   if (Channel->Ca() >= CA_ENCRYPTED_MIN) {
@@ -284,8 +286,12 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
              continue; // no CAM is able to decrypt this channel and the device uses vdr handled CAMs
           if (NumUsableSlots && !HasInternalCam && !CamSlots.Get(j)->Assign(device[i], true))
              continue; // CAM slot can't be used with this device
-          bool ndr;
-          if (device[i]->ProvidesChannel(Channel, Priority, &ndr)) { // this device is basically able to do the job
+          bool ndr = false;
+          bool TunedToTransponder = device[i]->IsTunedToTransponder(Channel);
+          if (TunedToTransponder || device[i]->ProvidesChannel(Channel, Priority, &ndr)) { // this device is basically able to do the job
+             bool OccupiedOtherTransponder = !TunedToTransponder && device[i]->Occupied();
+             if (OccupiedOtherTransponder)
+                ndr = true;
              if (NumUsableSlots && !HasInternalCam) {
                 if (cCamSlot *csi = device[i]->CamSlot()) {
                    cCamSlot *csj = CamSlots.Get(j);
@@ -303,10 +309,10 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
              imp <<= 1; imp |= (LiveView && NumUsableSlots && !HasInternalCam) ? !ChannelCamRelations.CamDecrypt(Channel->GetChannelID(), CamSlots.Get(j)->MasterSlotNumber()) || ndr : 0; // prefer CAMs that are known to decrypt this channel for live viewing, if we don't need to detach existing receivers
              imp <<= 1; imp |= LiveView ? !device[i]->IsPrimaryDevice() || ndr : 0;                                  // prefer the primary device for live viewing if we don't need to detach existing receivers
              imp <<= 1; imp |= !device[i]->Receiving() && (device[i] != cTransferControl::ReceiverDevice() || device[i]->IsPrimaryDevice()) || ndr; // use receiving devices if we don't need to detach existing receivers, but avoid primary device in local transfer mode
-             imp <<= 1; imp |= device[i]->Receiving();                                                               // avoid devices that are receiving
+             imp <<= 1; imp |= device[i]->Receiving() || OccupiedOtherTransponder;                                   // avoid devices that are receiving
              imp <<= 5; imp |= GetClippedNumProvidedSystems(5, device[i]) - 1;                                       // avoid cards which support multiple delivery systems
-             imp <<= 1; imp |= device[i] == cTransferControl::ReceiverDevice();                                      // avoid the Transfer Mode receiver device
              imp <<= 8; imp |= device[i]->Priority() - IDLEPRIORITY;                                                 // use the device with the lowest priority (- IDLEPRIORITY to assure that values -100..99 can be used)
+             imp <<= 1; imp |= device[i] == cTransferControl::ReceiverDevice();                                      // avoid the Transfer Mode receiver device
              imp <<= 8; imp |= ((NumUsableSlots && !HasInternalCam) ? SlotPriority[j] : IDLEPRIORITY) - IDLEPRIORITY;// use the CAM slot with the lowest priority (- IDLEPRIORITY to assure that values -100..99 can be used)
              imp <<= 1; imp |= ndr;                                                                                  // avoid devices if we need to detach existing receivers
              imp <<= 1; imp |= (NumUsableSlots || InternalCamNeeded) ? 0 : device[i]->HasCi();                       // avoid cards with Common Interface for FTA channels
@@ -321,6 +327,7 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
                 if (NumUsableSlots && !HasInternalCam)
                    s = CamSlots.Get(j);
                 }
+             //dsyslog("device %d provides channel %d prio %d ndr %d imp %.8X", device[i]->DeviceNumber() + 1, Channel->Number(), Priority, ndr, imp);
              }
           }
       if (!NumUsableSlots)
@@ -427,7 +434,8 @@ cDevice *cDevice::GetDeviceForTransponder(const cChannel *Channel, int Priority)
          if (d->ProvidesTransponder(Channel)) {
             if (d->MaySwitchTransponder(Channel))
                return d; // this device may switch to the transponder without disturbing any receiver or live view
-            else if (!d->Occupied() && !d->IsBonded()) { // MaySwitchTransponder() implicitly calls Occupied()
+            else if (!d->Occupied(Priority) && !d->IsBonded() && d->Priority(true) < LIVEPRIORITY) { // MaySwitchTransponder() implicitly calls Occupied()
+               // we select only devices with priority < LIVEPRIORITY, so device can be switched without impact on recordings or live viewing
                if (d->Priority() < Priority && (!Device || d->Priority() < Device->Priority()))
                   Device = d; // use this one only if no other with less impact can be found
                }
@@ -437,6 +445,12 @@ cDevice *cDevice::GetDeviceForTransponder(const cChannel *Channel, int Priority)
   return Device;
 }
 
+void cDevice::ReleaseCamSlot(void)
+{
+  if (camSlot && !camSlot->IsDecrypting() && !camSlot->IsActivating())
+     camSlot->Assign(NULL);
+}
+
 bool cDevice::HasCi(void)
 {
   return false;
@@ -667,11 +681,11 @@ void cDevice::StartSectionHandler(void)
 void cDevice::StopSectionHandler(void)
 {
   if (sectionHandler) {
+     delete sectionHandler; // automatically detaches filters
      delete nitFilter;
      delete sdtFilter;
      delete patFilter;
      delete eitFilter;
-     delete sectionHandler;
      nitFilter = NULL;
      sdtFilter = NULL;
      patFilter = NULL;
@@ -795,7 +809,17 @@ bool cDevice::IsTunedToTransponder(const cChannel *Channel) const
 
 bool cDevice::MaySwitchTransponder(const cChannel *Channel) const
 {
-  return time(NULL) > occupiedTimeout && !Receiving() && !(pidHandles[ptAudio].pid || pidHandles[ptVideo].pid || pidHandles[ptDolby].pid);
+  return !Occupied() && !Receiving() && !(pidHandles[ptAudio].pid || pidHandles[ptVideo].pid || pidHandles[ptDolby].pid);
+}
+
+void cDevice::SetPowerSaveMode(bool On)
+{
+}
+
+void cDevice::SetPowerSaveIfUnused(void)
+{
+  if (!Occupied() && !Receiving() && !(pidHandles[ptAudio].pid || pidHandles[ptVideo].pid || pidHandles[ptDolby].pid))
+     SetPowerSaveMode(true);
 }
 
 bool cDevice::SwitchChannel(const cChannel *Channel, bool LiveView)
@@ -870,7 +894,7 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
      DELETENULL(dvbSubtitleConverter);
      }
 
-  cDevice *Device = (LiveView && IsPrimaryDevice()) ? GetDevice(Channel, LIVEPRIORITY, true) : this;
+  cDevice *Device = (LiveView && IsPrimaryDevice(false)) ? GetDevice(Channel, LIVEPRIORITY, true) : this;
 
   bool NeedsTransferMode = LiveView && Device != PrimaryDevice();
   // If the CAM slot wants the TS data, we need to switch to Transfer Mode:
@@ -902,6 +926,7 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
      // channel to it, for possible later decryption:
      if (camSlot)
         camSlot->AddChannel(Channel);
+     SetPowerSaveMode(false);
      if (SetChannelDevice(Channel, LiveView)) {
         // Start section handling:
         if (sectionHandler) {
@@ -917,23 +942,26 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
      }
 
   if (Result == scrOk) {
-     if (LiveView && IsPrimaryDevice()) {
-        if (patFilter) // this is only for FF DVB cards!
-           patFilter->Request(Channel->Sid());
-        currentChannel = Channel->Number();
-        // Set the available audio tracks:
-        ClrAvailableTracks();
-        for (int i = 0; i < MAXAPIDS; i++)
-            SetAvailableTrack(ttAudio, i, Channel->Apid(i), Channel->Alang(i));
-        if (Setup.UseDolbyDigital) {
-           for (int i = 0; i < MAXDPIDS; i++)
-               SetAvailableTrack(ttDolby, i, Channel->Dpid(i), Channel->Dlang(i));
+     if (LiveView) {
+        if (IsPrimaryDevice(false))
+           currentChannel = Channel->Number();
+        if (IsPrimaryDevice()) {
+           if (patFilter) // this is only for FF DVB cards!
+              patFilter->Request(Channel->Sid());
+           // Set the available audio tracks:
+           ClrAvailableTracks();
+           for (int i = 0; i < MAXAPIDS; i++)
+               SetAvailableTrack(ttAudio, i, Channel->Apid(i), Channel->Alang(i));
+           if (Setup.UseDolbyDigital) {
+              for (int i = 0; i < MAXDPIDS; i++)
+                  SetAvailableTrack(ttDolby, i, Channel->Dpid(i), Channel->Dlang(i));
+              }
+           for (int i = 0; i < MAXSPIDS; i++)
+               SetAvailableTrack(ttSubtitle, i, Channel->Spid(i), Channel->Slang(i));
+           if (!NeedsTransferMode)
+              EnsureAudioTrack(true);
+           EnsureSubtitleTrack();
            }
-        for (int i = 0; i < MAXSPIDS; i++)
-            SetAvailableTrack(ttSubtitle, i, Channel->Spid(i), Channel->Slang(i));
-        if (!NeedsTransferMode)
-           EnsureAudioTrack(true);
-        EnsureSubtitleTrack();
         }
      cStatus::MsgChannelSwitch(this, Channel->Number(), LiveView); // only report status if channel switch successful
      }
@@ -945,21 +973,34 @@ void cDevice::ForceTransferMode(void)
 {
   if (!cTransferControl::ReceiverDevice()) {
      LOCK_CHANNELS_READ;
-     if (const cChannel *Channel = Channels->GetByNumber(CurrentChannel()))
+     if (const cChannel *Channel = Channels->GetByNumber(CurrentChannel())) {
+        SetPowerSaveMode(false);
         SetChannelDevice(Channel, false); // this implicitly starts Transfer Mode
+        }
      }
 }
 
-int cDevice::Occupied(void) const
+int cDevice::Occupied(int Priority) const
 {
+  if (Priority > occupiedPriority)
+     return 0;
   int Seconds = occupiedTimeout - time(NULL);
   return Seconds > 0 ? Seconds : 0;
 }
 
-void cDevice::SetOccupied(int Seconds)
+void cDevice::SetOccupied(int Seconds, int Priority, time_t From)
 {
-  if (Seconds >= 0)
-     occupiedTimeout = time(NULL) + min(Seconds, MAXOCCUPIEDTIMEOUT);
+  if (Seconds < 0)
+     return;
+  if (From == 0)
+     From = time(NULL);
+  if (From == occupiedFrom)
+     occupiedPriority = max(Priority, occupiedPriority);
+  else {
+     occupiedPriority = Priority;
+     occupiedFrom = From;
+     }
+  occupiedTimeout = From + min(Seconds, MAXOCCUPIEDTIMEOUT);
 }
 
 bool cDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
@@ -1588,7 +1629,6 @@ int cDevice::PlayTsSubtitle(const uchar *Data, int Length)
   return Length;
 }
 
-//TODO detect and report continuity errors?
 int cDevice::PlayTs(const uchar *Data, int Length, bool VideoOnly)
 {
   int Played = 0;
@@ -1652,11 +1692,13 @@ int cDevice::PlayTs(const uchar *Data, int Length, bool VideoOnly)
   return Played;
 }
 
-int cDevice::Priority(void) const
+int cDevice::Priority(bool IgnoreOccupied) const
 {
   int priority = IDLEPRIORITY;
   if (IsPrimaryDevice() && !Replaying() && HasProgramme())
      priority = TRANSFERPRIORITY; // we use the same value here, no matter whether it's actual Transfer Mode or real live viewing
+  if (!IgnoreOccupied && time(NULL) <= occupiedTimeout && occupiedPriority > priority)
+     priority = occupiedPriority - 1; // so timers with occupiedPriority can start
   cMutexLock MutexLock(&mutexReceiver);
   for (int i = 0; i < MAXRECEIVERS; i++) {
       if (receiver[i])
@@ -1824,7 +1866,7 @@ bool cDevice::AttachReceiver(cReceiver *Receiver)
   return false;
 }
 
-void cDevice::Detach(cReceiver *Receiver)
+void cDevice::Detach(cReceiver *Receiver, bool ReleaseCam)
 {
   if (!Receiver || Receiver->device != this)
      return;
@@ -1846,8 +1888,8 @@ void cDevice::Detach(cReceiver *Receiver)
   if (camSlot) {
      if (Receiver->priority > MINPRIORITY) { // priority check to avoid an infinite loop with the CAM slot's caPidReceiver
         camSlot->StartDecrypting();
-        if (!camSlot->IsDecrypting() && !camSlot->IsActivating())
-           camSlot->Assign(NULL);
+        if (ReleaseCam)
+           ReleaseCamSlot();
         }
      }
   if (!receiversLeft)
@@ -1861,8 +1903,9 @@ void cDevice::DetachAll(int Pid)
      for (int i = 0; i < MAXRECEIVERS; i++) {
          cReceiver *Receiver = receiver[i];
          if (Receiver && Receiver->WantsPid(Pid))
-            Detach(Receiver);
+            Detach(Receiver, false);
          }
+     ReleaseCamSlot();
      }
 }
 
@@ -1870,7 +1913,8 @@ void cDevice::DetachAllReceivers(void)
 {
   cMutexLock MutexLock(&mutexReceiver);
   for (int i = 0; i < MAXRECEIVERS; i++)
-      Detach(receiver[i]);
+      Detach(receiver[i], false);
+  ReleaseCamSlot();
 }
 
 // --- cTSBuffer -------------------------------------------------------------
