@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h 5.1 2021/05/21 12:51:16 kls Exp $
+ * $Id: device.h 5.5 2024/07/06 11:19:21 kls Exp $
  */
 
 #ifndef __DEVICE_H
@@ -178,6 +178,9 @@ public:
          ///< the transponder of the given Channel, without disturbing any receiver
          ///< at priorities higher or equal to Priority.
          ///< If no such device is currently available, NULL will be returned.
+         ///< Devices recording (Device->Priority(true) >= LIVEPRIORITY) will not be returned,
+         ///< even if Priority >= LIVEPRIORITY. Such higher priorities are only used to
+         ///< override occupied.
   static void Shutdown(void);
          ///< Closes down all devices.
          ///< Must be called at the end of the program.
@@ -217,7 +220,7 @@ protected:
          ///< May be dropped in a future version, if a better solution is found.
          ///< Do not use otherwise!
 public:
-  bool IsPrimaryDevice(void) const { return this == primaryDevice && HasDecoder(); }
+  bool IsPrimaryDevice(bool CheckDecoder = true) const { return this == primaryDevice && (!CheckDecoder || HasDecoder()); }
   int CardIndex(void) const { return cardIndex; }
          ///< Returns the card index of this device (0 ... MAXDEVICES - 1).
   int DeviceNumber(void) const;
@@ -259,7 +262,9 @@ public:
 
 private:
   mutable cMutex mutexChannel;
+  time_t occupiedFrom;
   time_t occupiedTimeout;
+  int occupiedPriority;
 protected:
   static int currentChannel;
 public:
@@ -341,6 +346,14 @@ public:
          ///< device, without disturbing any other activities. If an occupied timeout
          ///< has been set for this device, and that timeout has not yet expired,
          ///< this function returns false.
+  virtual void SetPowerSaveMode(bool On);
+         ///< Puts the device into power save mode, if applicable.
+         ///< If On is true, power save mode shall be activated, if it is false,
+         ///< normal operating mode shall be restored.
+         ///< The default implementation does nothing.
+  void SetPowerSaveIfUnused(void);
+         ///< Sets this device into a power save mode if it is not currently used and
+         ///< has implemented SetPowerSaveMode().
   bool SwitchChannel(const cChannel *Channel, bool LiveView);
          ///< Switches the device to the given Channel, initiating transfer mode
          ///< if necessary.
@@ -369,9 +382,10 @@ public:
          ///< channel number while replaying.
   void ForceTransferMode(void);
          ///< Forces the device into transfermode for the current channel.
-  int Occupied(void) const;
-         ///< Returns the number of seconds this device is still occupied for.
-  void SetOccupied(int Seconds);
+  int Occupied(int Priority = MINPRIORITY) const;
+         ///< Returns the number of seconds this device is still occupied for
+         ///< with a priority >= Priority.
+  void SetOccupied(int Seconds, int Priority = MINPRIORITY, time_t From = 0);
          ///< Sets the occupied timeout for this device to the given number of
          ///< Seconds, This can be used to tune a device to a particular transponder
          ///< and make sure it will stay there for a certain amount of time, for
@@ -379,6 +393,10 @@ public:
          ///< after the device has been successfully tuned to the requested transponder.
          ///< Seconds will be silently limited to MAXOCCUPIEDTIMEOUT. Values less than
          ///< 0 will be silently ignored.
+         ///< The timeout is counted from the given From time (by default the current time).
+         ///< Calling this function several times with the same From time will set the
+         ///< priority to the maximum of the given values.
+         ///< Priority() may return a value >= Priority until the timeout.
   virtual bool HasLock(int TimeoutMs = 0) const;
          ///< Returns true if the device has a lock on the requested transponder.
          ///< Default is true, a specific device implementation may return false
@@ -465,6 +483,8 @@ public:
 
 private:
   cCamSlot *camSlot;
+  void ReleaseCamSlot(void);
+         ///< Releases the CAM slot if it is currently not used.
 public:
   virtual bool HasCi(void);
          ///< Returns true if this device has a Common Interface.
@@ -831,9 +851,10 @@ private:
   mutable cMutex mutexReceiver;
   cReceiver *receiver[MAXRECEIVERS];
 public:
-  int Priority(void) const;
+  int Priority(bool IgnoreOccupied = false) const;
       ///< Returns the priority of the current receiving session (-MAXPRIORITY..MAXPRIORITY),
       ///< or IDLEPRIORITY if no receiver is currently active.
+      ///< If IgnoreOccupied is true, a priority set with SetOccupied() is ignored.
 protected:
   virtual bool OpenDvr(void);
       ///< Opens the DVR of this device and prepares it to deliver a Transport
@@ -852,8 +873,11 @@ public:
        ///< Returns true if we are currently receiving. The parameter has no meaning (for backwards compatibility only).
   bool AttachReceiver(cReceiver *Receiver);
        ///< Attaches the given receiver to this device.
-  void Detach(cReceiver *Receiver);
+  void Detach(cReceiver *Receiver, bool ReleaseCam = true);
        ///< Detaches the given receiver from this device.
+       ///< If ReleaseCam is true, the CAM slot will be released if it
+       ///< is no longer used. Otherwise the caller must call ReleaseCamSlot() after
+       ///< calling this function.
   void DetachAll(int Pid);
        ///< Detaches all receivers from this device for this pid.
   virtual void DetachAllReceivers(void);
