@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.c 5.1 2021/06/09 09:12:25 kls Exp $
+ * $Id: dvbdevice.c 5.7 2024/07/14 15:41:01 kls Exp $
  */
 
 #include "dvbdevice.h"
@@ -23,6 +23,8 @@
 
 static int DvbApiVersion = 0x0000; // the version of the DVB driver actually in use (will be determined by the first device created)
 
+#define BANDWIDTH_HZ_AUTO 0 // missing in DVB API 5
+
 #define DVBS_TUNE_TIMEOUT  9000 //ms
 #define DVBS_LOCK_TIMEOUT  2000 //ms
 #define DVBC_TUNE_TIMEOUT  9000 //ms
@@ -59,6 +61,7 @@ const tDvbParameterMap BandwidthValues[] = {
   {    8,  8000000, "8 MHz" },
   {   10, 10000000, "10 MHz" },
   { 1712,  1712000, "1.712 MHz" },
+  {  999, BANDWIDTH_HZ_AUTO, trNOOP("auto") },
   {  -1, 0, NULL }
   };
 
@@ -255,10 +258,10 @@ bool cDvbTransponderParameters::Parse(const char *s)
 {
   polarization = 0;
   inversion    = INVERSION_AUTO;
-  bandwidth    = 8000000;
+  bandwidth    = BANDWIDTH_HZ_AUTO;
   coderateH    = FEC_AUTO;
   coderateL    = FEC_AUTO;
-  modulation   = QPSK;
+  modulation   = QAM_AUTO;
   system       = DVB_SYSTEM_1;
   transmission = TRANSMISSION_MODE_AUTO;
   guard        = GUARD_INTERVAL_AUTO;
@@ -546,12 +549,12 @@ private:
   mutable uint32_t lastUncDelta;
   mutable time_t lastUncChange;
   cChannel channel;
-  const cDiseqc *lastDiseqc;
+  mutable const cDiseqc *lastDiseqc;
   int diseqcOffset;
-  int lastSource;
+  mutable int lastSource;
   cPositioner *positioner;
   const cScr *scr;
-  bool lnbPowerTurnedOn;
+  mutable bool lnbPowerTurnedOn;
   eTunerStatus tunerStatus;
   mutable cMutex mutex;
   cCondVar locked;
@@ -590,6 +593,7 @@ public:
   bool GetSignalStats(int &Valid, double *Strength = NULL, double *Cnr = NULL, double *BerPre = NULL, double *BerPost = NULL, double *Per = NULL, int *Status = NULL) const;
   int GetSignalStrength(void) const;
   int GetSignalQuality(void) const;
+  void SetPowerSaveMode(bool On);
   };
 
 cMutex cDvbTuner::bondMutex;
@@ -693,9 +697,12 @@ bool cDvbTuner::ProvidesFrontend(const cChannel *Channel, bool Activate) const
             fd_frontend = dvbFrontend->Open();
             frontend = i;
             dsyslog("using frontend %d/%d", adapter, frontend);
+            lastDiseqc = NULL;
+            lastSource = 0;
             lastUncValue = 0;
             lastUncDelta = 0;
             lastUncChange = 0;
+            lnbPowerTurnedOn = false;
             }
          return true;
          }
@@ -856,6 +863,8 @@ void cDvbTuner::ClearEventQueue(void) const
 
 bool cDvbTuner::GetFrontendStatus(fe_status_t &Status) const
 {
+  if (fd_frontend == -1)
+     return false;
   ClearEventQueue();
   Status = (fe_status_t)0; // initialize here to fix buggy drivers
   while (1) {
@@ -873,6 +882,8 @@ bool cDvbTuner::GetFrontendStatus(fe_status_t &Status) const
 
 bool cDvbTuner::GetSignalStats(int &Valid, double *Strength, double *Cnr, double *BerPre, double *BerPost, double *Per, int *Status) const
 {
+  if (fd_frontend == -1)
+     return false;
   ClearEventQueue();
   fe_status_t FeStatus = (fe_status_t)0; // initialize here to fix buggy drivers
   dtv_property Props[MAXFRONTENDCMDS];
@@ -1223,6 +1234,8 @@ int SignalToSQI(const cChannel *Channel, int Signal, int Ber, int FeModulation,
 
 int cDvbTuner::GetSignalStrength(void) const
 {
+  if (fd_frontend == -1)
+     return 0;
   ClearEventQueue();
   // Try DVB API 5:
   for (int i = 0; i < 1; i++) { // just a trick to break out with 'continue' ;-)
@@ -1288,6 +1301,8 @@ int cDvbTuner::GetSignalStrength(void) const
 
 int cDvbTuner::GetSignalQuality(void) const
 {
+  if (fd_frontend == -1)
+     return 0;
   // Try DVB API 5:
   for (int i = 0; i < 1; i++) { // just a trick to break out with 'continue' ;-)
       dtv_property Props[MAXFRONTENDCMDS];
@@ -1533,6 +1548,8 @@ void cDvbTuner::ExecuteDiseqc(const cDiseqc *Diseqc, int *Frequency)
 
 void cDvbTuner::ResetToneAndVoltage(void)
 {
+  if (fd_frontend == -1)
+     return;
   CHECK(ioctl(fd_frontend, FE_SET_VOLTAGE, bondedTuner ? SEC_VOLTAGE_OFF : SEC_VOLTAGE_13));
   CHECK(ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF));
 }
@@ -1765,6 +1782,32 @@ void cDvbTuner::Action(void)
         }
 }
 
+void cDvbTuner::SetPowerSaveMode(bool On)
+{
+  cMutexLock MutexLock(&mutex);
+  if (On) {
+     if (fd_frontend != -1) {
+        dsyslog("closing frontend %d/%d", adapter, frontend);
+        tunerStatus = tsIdle;
+        dvbFrontend->Close();
+        fd_frontend = -1;
+        channel = cChannel();
+        }
+     }
+  else {
+     if (fd_frontend == -1) {
+        dsyslog("opening frontend %d/%d", adapter, frontend);
+        fd_frontend = dvbFrontend->Open();
+        lastDiseqc = NULL;
+        lastSource = 0;
+        lastUncValue = 0;
+        lastUncDelta = 0;
+        lastUncChange = 0;
+        lnbPowerTurnedOn = false;
+        }
+     }
+}
+
 // --- cDvbSourceParam -------------------------------------------------------
 
 class cDvbSourceParam : public cSourceParam {
@@ -1855,16 +1898,16 @@ cDvbDevice::cDvbDevice(int Adapter, int Frontend)
 
   // We only check the devices that must be present - the others will be checked before accessing them://XXX
 
-  dvbTuner = new cDvbTuner(this, adapter, frontend);
-
   StartSectionHandler();
+
+  dvbTuner = new cDvbTuner(this, adapter, frontend);
 }
 
 cDvbDevice::~cDvbDevice()
 {
-  StopSectionHandler();
   delete dvbTuner;
   delete ciAdapter;
+  StopSectionHandler();
   UnBond();
   // We're not explicitly closing any device files here, since this sometimes
   // caused segfaults. Besides, the program is about to terminate anyway...
@@ -2297,6 +2340,11 @@ bool cDvbDevice::MaySwitchTransponder(const cChannel *Channel) const
   return BondingOk(Channel, true) && cDevice::MaySwitchTransponder(Channel);
 }
 
+void cDvbDevice::SetPowerSaveMode(bool On)
+{
+  dvbTuner->SetPowerSaveMode(On);
+}
+
 bool cDvbDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
 {
   if (dvbTuner->ProvidesFrontend(Channel, true)) {
