@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.h 4.4.1.2 2019/03/10 12:18:02 kls Exp $
+ * $Id: dvbdevice.h 5.1 2024/07/08 09:34:33 kls Exp $
  */
 
 #ifndef __DVBDEVICE_H
@@ -179,6 +179,7 @@ public:
          ///< Returns true if any devices are available.
 protected:
   int adapter, frontend;
+  virtual bool IsBonded(void) const { return bondedDevice; }
 private:
   int fd_dvr, fd_ca;
   bool checkTsBuffer;
@@ -243,6 +244,7 @@ public:
   virtual const cChannel *GetCurrentlyTunedTransponder(void) const;
   virtual bool IsTunedToTransponder(const cChannel *Channel) const;
   virtual bool MaySwitchTransponder(const cChannel *Channel) const;
+  virtual void SetPowerSaveMode(bool On);
 protected:
   virtual bool SetChannelDevice(const cChannel *Channel, bool LiveView);
 public:
