@@ -7,7 +7,7 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.c 5.6 2021/04/29 09:19:58 kls Exp $
+ * $Id: epg.c 5.9 2024/06/21 06:27:20 kls Exp $
  */
 
 #include "epg.h"
@@ -177,7 +177,7 @@ void cEvent::SetVersion(uchar Version)
 void cEvent::SetRunningStatus(int RunningStatus, const cChannel *Channel)
 {
   if (Channel && runningStatus != RunningStatus && (RunningStatus > SI::RunningStatusNotRunning || runningStatus > SI::RunningStatusUndefined) && schedule && schedule->HasTimer())
-     isyslog("channel %d (%s) event %s status %d", Channel->Number(), Channel->Name(), *ToDescr(), RunningStatus);
+     isyslog("channel %d (%s) event %s status %d->%d", Channel->Number(), Channel->Name(), *ToDescr(), runningStatus, RunningStatus);
   runningStatus = RunningStatus;
 }
 
@@ -451,7 +451,7 @@ cString cEvent::GetVpsString(void) const
 void cEvent::Dump(FILE *f, const char *Prefix, bool InfoOnly) const
 {
   if (InfoOnly || startTime + duration + EPG_LINGER_TIME >= time(NULL)) {
-     fprintf(f, "%sE %u %ld %d %X %X\n", Prefix, eventID, startTime, duration, tableID, version);
+     fprintf(f, "%sE %u %jd %d %X %X\n", Prefix, eventID, intmax_t(startTime), duration, tableID, version);
      if (!isempty(title))
         fprintf(f, "%sT %s\n", Prefix, title);
      if (!isempty(shortText))
@@ -476,7 +476,7 @@ void cEvent::Dump(FILE *f, const char *Prefix, bool InfoOnly) const
             }
         }
      if (vps)
-        fprintf(f, "%sV %ld\n", Prefix, vps);
+        fprintf(f, "%sV %jd\n", Prefix, intmax_t(vps));
      if (!InfoOnly && !isempty(aux)) {
         strreplace(aux, '\n', '|');
         fprintf(f, "%s@ %s\n", Prefix, aux);
@@ -518,7 +518,7 @@ bool cEvent::Parse(char *s)
                  components = new cComponents;
               components->SetComponent(components->NumComponents(), t);
               break;
-    case 'V': SetVps(atoi(t));
+    case 'V': SetVps(atol(t));
               break;
     case '@': strreplace(t, '|', '\n');
               SetAux(t);
@@ -541,11 +541,11 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule, int &Line)
            switch (*s) {
              case 'E': if (!Event) {
                           unsigned int EventID;
-                          time_t StartTime;
+                          intmax_t StartTime; // actually time_t, but intmax_t for scanning with "%jd"
                           int Duration;
                           unsigned int TableID = 0;
                           unsigned int Version = 0xFF; // actual value is ignored
-                          int n = sscanf(t, "%u %ld %d %X %X", &EventID, &StartTime, &Duration, &TableID, &Version);
+                          int n = sscanf(t, "%u %jd %d %X %X", &EventID, &StartTime, &Duration, &TableID, &Version);
                           if (n >= 3 && n <= 5) {
                              Event = (cEvent *)Schedule->GetEventByTime(StartTime);
                              cEvent *newEvent = NULL;
@@ -885,6 +885,23 @@ void cEvent::FixEpgBugs(void)
 
 Final:
 
+  // And then there are the specially gifted people who put a literal "\n" string where there should be
+  // a '\n' character:
+  if (shortText) {
+     if (char *p = strstr(shortText, "\\n")) {
+        *p = 0;
+        p += 2;
+        char *s = strdup(shortText);
+        char *d = strdup(cString::sprintf("%s\n\n%s", p, description));
+        free(shortText);
+        free(description);
+        shortText = s;
+        description = d;
+        EpgBugFixStat(12, ChannelID());
+        }
+     }
+  description = strreplace(description, "\\n", " \n");
+
   // VDR can't usefully handle newline characters in the title, shortText or component description of EPG
   // data, so let's always convert them to blanks (independent of the setting of EPGBugfixLevel):
   strreplace(title, '\n', ' ');
