@@ -6,7 +6,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: descriptor.c 4.0.1.1 2019/03/15 22:38:36 kls Exp $
+ *   $Id: descriptor.c 4.4 2020/12/17 12:52:57 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -90,17 +90,22 @@ char *ExtendedEventDescriptors::getText(const char *separation1, const char *sep
 }
 
 char *ExtendedEventDescriptors::getText(char *buffer, int size, const char *separation1, const char *separation2) {
+   int tmpsize = size;
+   char tmpbuf[tmpsize];
+   const char *fromCode = NULL;
    int index=0, len;
+   *tmpbuf = 0; // just in case length is 0
    for (int i=0;i<length;i++) {
       ExtendedEventDescriptor *d=(ExtendedEventDescriptor *)array[i];
       if (!d)
          continue;
-      d->text.getText(buffer+index, size);
-      len = strlen(buffer+index);
+      d->text.getText(tmpbuf+index, tmpsize, &fromCode);
+      len = strlen(tmpbuf+index);
       index += len;
-      size -= len;
+      tmpsize -= len;
    }
-
+   index = convertCharacterTable(tmpbuf, strlen(tmpbuf), buffer, size, fromCode);
+   size -= index;
    int sepLen1 = strlen(separation1);
    int sepLen2 = strlen(separation2);
    bool separated = false;
@@ -563,6 +568,10 @@ int ComponentDescriptor::getStreamContent() const {
    return s->stream_content;
 }
 
+int ComponentDescriptor::getStreamContentExt() const {
+   return s->stream_content_ext;
+}
+
 int ComponentDescriptor::getComponentType() const {
    return s->component_type;
 }
