@@ -10,7 +10,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   $Id: headers.h 4.0 2013/10/30 10:16:18 kls Exp $
+ *   $Id: headers.h 4.2 2020/09/16 13:48:33 kls Exp $
  *                                                                         *
  ***************************************************************************/
 
@@ -1176,11 +1176,11 @@ struct descr_component {
    u_char descriptor_tag                         :8;
    u_char descriptor_length                      :8;
 #if BYTE_ORDER == BIG_ENDIAN
-   u_char reserved                               :4;
+   u_char stream_content_ext                     :4;
    u_char stream_content                         :4;
 #else
    u_char stream_content                         :4;
-   u_char reserved                               :4;
+   u_char stream_content_ext                     :4;
 #endif
    u_char component_type                         :8;
    u_char component_tag                          :8;
@@ -2081,7 +2081,7 @@ struct descr_application_icons_descriptor_end {
    u_char icon_flags_lo                          :8;
 };
 
-/* 0x15 simple application location descrptor */
+/* 0x15 simple application location descriptor */
 
 #define DESCR_SIMPLE_APPLICATION_LOCATION_LEN 3
 
