@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.h 4.5.1.1 2019/05/24 21:30:20 kls Exp $
+ * $Id: osd.h 5.2 2024/01/18 12:04:57 kls Exp $
  */
 
 #ifndef __OSD_H
@@ -447,6 +447,11 @@ public:
        ///< Clears the image data by setting all pixels to be fully transparent.
   void Fill(tColor Color);
        ///< Fills the image data with the given Color.
+  cImage *Scaled(double FactorX, double FactorY, bool AntiAlias = false) const;
+       ///< Creates a copy of this image, scaled by the given factors.
+       ///< If AntiAlias is true and either of the factors is greater than 1.0,
+       ///< anti-aliasing is applied.
+       ///< The caller must delete the returned image once it is no longer used.
   };
 
 #define MAXPIXMAPLAYERS    8
@@ -602,11 +607,32 @@ public:
        ///< the given Point. ImageHandle must be a value that has previously been
        ///< returned by a call to cOsdProvider::StoreImage(). If ImageHandle
        ///< has an invalid value, nothing happens.
+  virtual void DrawScaledImage(const cPoint &Point, const cImage &Image, double FactorX, double FactorY, bool AntiAlias = false) {};
+       ///< Draws the given Image into this pixmap at the given Point and scales it.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< This function should be 'pure', just like all the others. However, since it was
+       ///< introduced when this class was already in widespread use, the default was made
+       ///< empty, so that existing code will still compile.
+  virtual void DrawScaledImage(const cPoint &Point, int ImageHandle, double FactorX, double FactorY, bool AntiAlias = false) {};
+       ///< Draws the image referenced by the given ImageHandle into this pixmap at
+       ///< the given Point and scales it. ImageHandle must be a value that has
+       ///< previously been returned by a call to cOsdProvider::StoreImage().
+       ///< If ImageHandle has an invalid value, nothing happens.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< This function should be 'pure', just like all the others. However, since it was
+       ///< introduced when this class was already in widespread use, the default was made
+       ///< empty, so that existing code will still compile.
   virtual void DrawPixel(const cPoint &Point, tColor Color) = 0;
        ///< Sets the pixel at the given Point to the given Color, which is
        ///< a full 32 bit ARGB value. If the alpha value of Color is not 0xFF
        ///< (fully opaque), and this is the background pixmap (layer 0), the pixel is
        ///< alpha blended with the existing color at the given position in this pixmap.
+  virtual void DrawBlendedPixel(const cPoint &Point, tColor Color, uint8_t AlphaLayer = ALPHA_OPAQUE) { DrawPixel(Point, Color); }
+       ///< Like DrawPixel(), but with an additional AlphaLayer, and works on any
+       ///< pixmap, not only the background. The default implementation just calls
+       ///< DrawPixel(), ignoring AlphaLayer.
   virtual void DrawBitmap(const cPoint &Point, const cBitmap &Bitmap, tColor ColorFg = 0, tColor ColorBg = 0, bool Overlay = false) = 0;
        ///< Sets the pixels in the OSD with the data from the given
        ///< Bitmap, putting the upper left corner of the Bitmap at Point.
@@ -696,7 +722,10 @@ public:
   virtual void Fill(tColor Color);
   virtual void DrawImage(const cPoint &Point, const cImage &Image);
   virtual void DrawImage(const cPoint &Point, int ImageHandle);
+  virtual void DrawScaledImage(const cPoint &Point, const cImage &Image, double FactorX, double FactorY, bool AntiAlias = false);
+  virtual void DrawScaledImage(const cPoint &Point, int ImageHandle, double FactorX, double FactorY, bool AntiAlias = false);
   virtual void DrawPixel(const cPoint &Point, tColor Color);
+  virtual void DrawBlendedPixel(const cPoint &Point, tColor Color, uint8_t AlphaLayer = ALPHA_OPAQUE);
   virtual void DrawBitmap(const cPoint &Point, const cBitmap &Bitmap, tColor ColorFg = 0, tColor ColorBg = 0, bool Overlay = false);
   virtual void DrawText(const cPoint &Point, const char *s, tColor ColorFg, tColor ColorBg, const cFont *Font, int Width = 0, int Height = 0, int Alignment = taDefault);
   virtual void DrawRectangle(const cRect &Rect, tColor Color);
@@ -785,12 +814,6 @@ protected:
        ///< If there are no dirty pixmaps, or if this is not a true color OSD,
        ///< this function returns NULL.
        ///< The caller must call DestroyPixmap() for the returned pixmap after use.
-#ifndef DEPRECATED_GETBITMAP
-#define DEPRECATED_GETBITMAP 0
-#endif
-#if DEPRECATED_GETBITMAP
-public:
-#endif
   cBitmap *GetBitmap(int Area);
        ///< Returns a pointer to the bitmap for the given Area, or NULL if no
        ///< such bitmap exists.
@@ -858,6 +881,19 @@ public:
        ///< returned by a call to cOsdProvider::StoreImage(). If ImageHandle
        ///< has an invalid value, nothing happens.
        ///< If this is not a true color OSD, this function does nothing.
+  virtual void DrawScaledImage(const cPoint &Point, const cImage &Image, double FactorX, double FactorY, bool AntiAlias = false);
+       ///< Draws the given Image on this OSD at the given Point and scales it.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< If this is not a true color OSD, this function does nothing.
+  virtual void DrawScaledImage(const cPoint &Point, int ImageHandle, double FactorX, double FactorY, bool AntiAlias = false);
+       ///< Draws the image referenced by the given ImageHandle on this OSD at
+       ///< the given Point and scales it. ImageHandle must be a value that has
+       ///< previously been returned by a call to cOsdProvider::StoreImage().
+       ///< If ImageHandle has an invalid value, nothing happens.
+       ///< If AntiAlias is true and either of the factors is greater than
+       ///< 1.0, anti-aliasing is applied.
+       ///< If this is not a true color OSD, this function does nothing.
   virtual eOsdError CanHandleAreas(const tArea *Areas, int NumAreas);
        ///< Checks whether the OSD can display the given set of sub-areas.
        ///< The return value indicates whether a call to SetAreas() with this
