@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: pat.c 4.3.1.2 2019/03/15 13:13:40 kls Exp $
+ * $Id: pat.c 5.4 2021/06/21 20:13:55 kls Exp $
  */
 
 #include "pat.h"
@@ -280,51 +280,219 @@ int GetPmtPid(int Source, int Transponder, int ServiceId)
   return CaDescriptorHandler.GetPmtPid(Source, Transponder, ServiceId);
 }
 
+// --- cPmtPidEntry ----------------------------------------------------------
+
+class cPmtPidEntry : public cListObject {
+private:
+  int pid;
+  int count; // the number of SIDs currently requested from this PID
+  int state; // adding/deleting PIDs to/from the filter may only be done from within the Process() function,
+             // otherwise there could be a deadlock between cPatFilter::mutex and cSectionHandler::mutex;
+             // this member tells whether this PID needs to be added to (>0) or deleted from (<0) the filter
+  bool complete; // true if all SIDs on this PID have been received
+public:
+  cPmtPidEntry(int Pid);
+  int Pid(void) { return pid; }
+  int Count(void) { return count; }
+  int State(void) { int s = state; state = 0; return s; } // returns the current state and resets it
+  void SetState(int State) { state = State; } // 1 = add the PID, -1 = delete the PID, 0 = do nothing
+  void Inc(void) { if (++count == 1) state = 1; }
+  void Dec(void) { if (--count == 0) state = -1; }
+  int Complete(void) { return complete; }
+  void SetComplete(bool State) { complete = State; }
+  };
+
+cPmtPidEntry::cPmtPidEntry(int Pid)
+{
+  pid = Pid;
+  count = 0;
+  state = 0;
+  complete = false;
+}
+
+// --- cPmtSidEntry ----------------------------------------------------------
+
+class cPmtSidEntry : public cListObject {
+private:
+  int sid;
+  int pid;
+  cPmtPidEntry *pidEntry;
+  int version;
+  bool received;
+public:
+  cPmtSidEntry(int Sid, cPmtPidEntry *PidEntry);
+  int Sid(void) { return sid; }
+  int Pid(void) { return pid; }
+  cPmtPidEntry *PidEntry(void) { return pidEntry; }
+  int Version(void) { return version; }
+  int Received(void) { return received; }
+  void SetVersion(int Version) { version = Version; }
+  void SetReceived(bool State) { received = State; }
+  };
+
+cPmtSidEntry::cPmtSidEntry(int Sid, cPmtPidEntry *PidEntry)
+{
+  sid = Sid;
+  pid = PidEntry->Pid();
+  pidEntry = PidEntry;
+  version = -1;
+  received = false;
+}
+
+// --- cPmtSidRequest --------------------------------------------------------
+
+class cPmtSidRequest : public cListObject {
+private:
+  int sid;
+  int count; // the number of requests for this SID
+public:
+  cPmtSidRequest(int Sid) { sid = Sid; count = 1; }
+  int Sid(void) { return sid; }
+  int Count(void) { return count; }
+  void Inc(void) { count++; }
+  void Dec(void) { count--; }
+  };
+
 // --- cPatFilter ------------------------------------------------------------
 
 //#define DEBUG_PAT_PMT
 #ifdef DEBUG_PAT_PMT
 #define DBGLOG(a...) { cString s = cString::sprintf(a); fprintf(stderr, "%s\n", *s); dsyslog("%s", *s); }
 #else
-#define DBGLOG(a...)
+#define DBGLOG(a...) void()
 #endif
 
 cPatFilter::cPatFilter(void)
 {
-  Trigger(0);
+  patVersion = -1;
+  activePmt = NULL;
+  transponder = 0;
+  source = 0;
   Set(0x00, 0x00);  // PAT
 }
 
 void cPatFilter::SetStatus(bool On)
 {
   cMutexLock MutexLock(&mutex);
+  if (On) { // restart all requested PMT Pids
+     for (cPmtPidEntry *pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid))
+         pPid->SetState(pPid->Count() > 0);
+     if (activePmt && activePmt->Count() == 0) {
+        activePmt->SetState(1);
+        timer.Set(PMT_SCAN_TIMEOUT);
+        }
+     }
   DBGLOG("PAT filter set status %d", On);
   cFilter::SetStatus(On);
-  Trigger();
 }
 
-void cPatFilter::Trigger(int Sid)
+bool cPatFilter::TransponderChanged(void)
+{
+  if (source != Source() || transponder != Transponder()) {
+     DBGLOG("PAT filter transponder changed from %d/%d to %d/%d", source, transponder, Source(), Transponder());
+     source = Source();
+     transponder = Transponder();
+     return true;
+     }
+  return false;
+}
+
+void cPatFilter::Trigger(int)
 {
   cMutexLock MutexLock(&mutex);
+  DBGLOG("PAT filter trigger");
   patVersion = -1;
-  pmtIndex = -1;
-  numPmtEntries = 0;
-  if (Sid >= 0) {
-     sid = Sid;
-     DBGLOG("PAT filter trigger SID %d", Sid);
-     }
+  sectionSyncer.Reset();
+}
+
+void cPatFilter::Request(int Sid)
+{
+  cMutexLock MutexLock(&mutex);
+  DBGLOG("PAT filter request SID %d", Sid);
+  for (cPmtSidRequest *sr = pmtSidRequestList.First(); sr; sr = pmtSidRequestList.Next(sr)) {
+      if (sr->Sid() == Sid) {
+         sr->Inc();
+         DBGLOG("PAT filter add SID request %d (%d)", Sid, sr->Count());
+         return;
+         }
+      }
+  DBGLOG("PAT filter new SID request %d", Sid);
+  pmtSidRequestList.Add(new cPmtSidRequest(Sid));
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Sid() == Sid) {
+         cPmtPidEntry *pPid = se->PidEntry();
+         pPid->Inc();
+         DBGLOG("    PMT pid %5d  SID %5d (%d)", pPid->Pid(), se->Sid(), pPid->Count());
+         break;
+         }
+      }
+}
+
+void cPatFilter::Release(int Sid)
+{
+  cMutexLock MutexLock(&mutex);
+  DBGLOG("PAT filter release SID %d", Sid);
+  for (cPmtSidRequest *sr = pmtSidRequestList.First(); sr; sr = pmtSidRequestList.Next(sr)) {
+      if (sr->Sid() == Sid) {
+         sr->Dec();
+         DBGLOG("PAT filter del SID request %d (%d)", Sid, sr->Count());
+         if (sr->Count() == 0) {
+            pmtSidRequestList.Del(sr);
+            for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+                if (se->Sid() == Sid) {
+                   cPmtPidEntry *pPid = se->PidEntry();
+                   pPid->Dec();
+                   DBGLOG("    PMT pid %5d  SID %5d (%d)", pPid->Pid(), se->Sid(), pPid->Count());
+                   break;
+                   }
+                }
+            }
+         break;
+         }
+      }
+}
+
+int cPatFilter::NumSidRequests(int Sid)
+{
+  for (cPmtSidRequest *sr = pmtSidRequestList.First(); sr; sr = pmtSidRequestList.Next(sr)) {
+      if (sr->Sid() == Sid)
+         return sr->Count();
+      }
+  return 0;
+}
+
+bool cPatFilter::PmtPidComplete(int PmtPid)
+{
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Pid() == PmtPid && !se->Received())
+         return false;
+      }
+  return true;
+}
+
+void cPatFilter::PmtPidReset(int PmtPid)
+{
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Pid() == PmtPid) {
+         se->SetReceived(false);
+         se->PidEntry()->SetComplete(false);
+         }
+      }
 }
 
 bool cPatFilter::PmtVersionChanged(int PmtPid, int Sid, int Version, bool SetNewVersion)
 {
-  int Id = MakePmtId(PmtPid, Sid);
-  for (int i = 0; i < numPmtEntries; i++) {
-      if (pmtId[i] == Id) {
-         if (pmtVersion[i] != Version) {
+  for (cPmtSidEntry *se = pmtSidList.First(); se; se = pmtSidList.Next(se)) {
+      if (se->Sid() == Sid && se->Pid() == PmtPid) {
+         if (!se->Received()) {
+            se->SetReceived(true);
+            se->PidEntry()->SetComplete(PmtPidComplete(PmtPid));
+            }
+         if (se->Version() != Version) {
             if (SetNewVersion)
-               pmtVersion[i] = Version;
+               se->SetVersion(Version);
             else
-               DBGLOG("PMT %d  %2d %5d %2d -> %2d", Transponder(), i, PmtPid, pmtVersion[i], Version);
+               DBGLOG("PMT %d  %5d/%5d  %2d -> %2d  %d", Transponder(), PmtPid, Sid, se->Version(), Version, NumSidRequests(Sid));
             return true;
             }
          break;
@@ -335,66 +503,109 @@ bool cPatFilter::PmtVersionChanged(int PmtPid, int Sid, int Version, bool SetNew
 
 void cPatFilter::SwitchToNextPmtPid(void)
 {
-  if (pmtIndex >= 0) {
-     Del(GetPmtPid(pmtIndex), SI::TableIdPMT);
-     pmtIndex = (pmtIndex + 1) % numPmtEntries;
-     Add(GetPmtPid(pmtIndex), SI::TableIdPMT);
+  if (activePmt) {
+     if (activePmt->Count() == 0)
+        Del(activePmt->Pid(), SI::TableIdPMT);
+     for (;;) {
+         activePmt = pmtPidList.Next(activePmt);
+         if (!activePmt || activePmt->Count() == 0)
+            break;
+         }
+     if (activePmt) {
+        PmtPidReset(activePmt->Pid());
+        Add(activePmt->Pid(), SI::TableIdPMT);
+        timer.Set(PMT_SCAN_TIMEOUT);
+        }
      }
 }
 
 void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
 {
   cMutexLock MutexLock(&mutex);
+  if (TransponderChanged()) {
+     patVersion = -1;
+     sectionSyncer.Reset();
+     }
+  if (patVersion >= 0) {
+     for (cPmtPidEntry *pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid)) {
+         int State = pPid->State();
+         if (State > 0)
+            Add(pPid->Pid(), SI::TableIdPMT);
+         else if (State < 0)
+            Del(pPid->Pid(), SI::TableIdPMT);
+         }
+     }
+  else if (Pid != 0x00)
+     return;
   if (Pid == 0x00) {
      if (Tid == SI::TableIdPAT) {
         SI::PAT pat(Data, false);
         if (!pat.CheckCRCAndParse())
            return;
-        if (pat.getVersionNumber() != patVersion) {
-           DBGLOG("PAT %d %d -> %d", Transponder(), patVersion, pat.getVersionNumber());
-           if (pmtIndex >= 0) {
-              Del(GetPmtPid(pmtIndex), SI::TableIdPMT);
-              pmtIndex = -1;
+        if (sectionSyncer.Check(pat.getVersionNumber(), pat.getSectionNumber())) {
+           DBGLOG("PAT %d %d -> %d %d/%d", Transponder(), patVersion, pat.getVersionNumber(), pat.getSectionNumber(), pat.getLastSectionNumber());
+           bool NeedsSetStatus = patVersion >= 0;
+           if (pat.getVersionNumber() != patVersion) {
+              if (NeedsSetStatus)
+                 SetStatus(false); // deletes all PIDs from the filter
+              activePmt = NULL;
+              pmtSidList.Clear();
+              pmtPidList.Clear();
+              patVersion = pat.getVersionNumber();
               }
-           numPmtEntries = 0;
            SI::PAT::Association assoc;
            for (SI::Loop::Iterator it; pat.associationLoop.getNext(assoc, it); ) {
-               if (!assoc.isNITPid() && numPmtEntries < MAXPMTENTRIES) {
-                  DBGLOG("    PMT pid %2d %5d  SID %5d", numPmtEntries, assoc.getPid(), assoc.getServiceId());
-                  pmtId[numPmtEntries] = MakePmtId(assoc.getPid(), assoc.getServiceId());
-                  pmtVersion[numPmtEntries] = -1;
-                  if (sid == assoc.getServiceId()) {
-                     pmtIndex = numPmtEntries;
-                     DBGLOG("sid = %d pmtIndex = %d", sid, pmtIndex);
+               if (!assoc.isNITPid()) {
+                  int PmtPid = assoc.getPid();
+                  int PmtSid = assoc.getServiceId();
+                  cPmtPidEntry *pPid = NULL;
+                  for (pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid)) {
+                      if (pPid->Pid() == PmtPid)
+                         break;
+                      }
+                  int SidRequest = NumSidRequests(PmtSid);
+                  DBGLOG("    PMT pid %5d  SID %5d%s%s", PmtPid, PmtSid, SidRequest ? " R" : "", pPid ? " S" : "");
+                  if (!pPid) { // new PMT Pid
+                     pPid = new cPmtPidEntry(PmtPid);
+                     pmtPidList.Add(pPid);
                      }
-                  numPmtEntries++;
+                  pmtSidList.Add(new cPmtSidEntry(PmtSid, pPid));
+                  if (SidRequest > 0)
+                     pPid->Inc();
                   }
                }
-           if (numPmtEntries > 0 && pmtIndex < 0)
-              pmtIndex = 0;
-           if (pmtIndex >= 0)
-              Add(GetPmtPid(pmtIndex), SI::TableIdPMT);
-           patVersion = pat.getVersionNumber();
-           timer.Set(PMT_SCAN_TIMEOUT);
+           if (sectionSyncer.Processed(pat.getSectionNumber(), pat.getLastSectionNumber())) { // all PAT sections done
+              for (cPmtPidEntry *pPid = pmtPidList.First(); pPid; pPid = pmtPidList.Next(pPid)) {
+                  if (pPid->Count() == 0) {
+                     pPid->SetState(1);
+                     activePmt = pPid;
+                     timer.Set(PMT_SCAN_TIMEOUT);
+                     break;
+                     }
+                  }
+              if (NeedsSetStatus)
+                 SetStatus(true);
+              }
            }
         }
      }
   else if (Tid == SI::TableIdPMT && Source() && Transponder()) {
-     timer.Set(PMT_SCAN_TIMEOUT);
      SI::PMT pmt(Data, false);
      if (!pmt.CheckCRCAndParse())
         return;
-     if (!PmtVersionChanged(Pid, pmt.getTableIdExtension(), pmt.getVersionNumber())) {
-        SwitchToNextPmtPid();
+     if (!PmtVersionChanged(Pid, pmt.getTableIdExtension(), pmt.getVersionNumber(), false)) {
+        if (activePmt && activePmt->Complete())
+           SwitchToNextPmtPid();
         return;
         }
      cStateKey StateKey;
      cChannels *Channels = cChannels::GetChannelsWrite(StateKey, 10);
      if (!Channels)
         return;
-     bool ChannelsModified = false;
      PmtVersionChanged(Pid, pmt.getTableIdExtension(), pmt.getVersionNumber(), true);
-     SwitchToNextPmtPid();
+     bool ChannelsModified = false;
+     if (activePmt && activePmt->Complete())
+        SwitchToNextPmtPid();
      cChannel *Channel = Channels->GetByServiceID(Source(), Transponder(), pmt.getServiceId());
      if (Channel) {
         SI::CaDescriptor *d;
@@ -636,9 +847,8 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
      StateKey.Remove(ChannelsModified);
      }
   if (timer.TimedOut()) {
-     if (pmtIndex >= 0)
-        DBGLOG("PMT timeout %d", pmtIndex);
+     if (activePmt)
+        DBGLOG("PMT timeout Pid %d", activePmt->Pid());
      SwitchToNextPmtPid();
-     timer.Set(PMT_SCAN_TIMEOUT);
      }
 }
