@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recorder.h 4.1 2015/09/05 11:46:23 kls Exp $
+ * $Id: recorder.h 5.1 2021/05/19 11:22:20 kls Exp $
  */
 
 #ifndef __RECORDER_H
@@ -16,19 +16,31 @@
 #include "ringbuffer.h"
 #include "thread.h"
 
+class cTsChecker;
+class cFrameChecker;
+
 class cRecorder : public cReceiver, cThread {
 private:
+  cTsChecker *tsChecker;
+  cFrameChecker *frameChecker;
   cRingBufferLinear *ringBuffer;
   cFrameDetector *frameDetector;
   cPatPmtGenerator patPmtGenerator;
   cFileName *fileName;
+  cRecordingInfo *recordingInfo;
   cIndexFile *index;
   cUnbufferedFile *recordFile;
   char *recordingName;
+  bool firstIframeSeen;
   off_t fileSize;
   time_t lastDiskSpaceCheck;
+  time_t lastErrorLog;
+  int oldErrors;
+  int errors;
+  int lastErrors;
   bool RunningLowOnDiskSpace(void);
   bool NextFile(void);
+  void HandleErrors(bool Force = false);
 protected:
   virtual void Activate(bool On);
        ///< If you override Activate() you need to call Detach() (which is a
@@ -42,6 +54,8 @@ public:
        ///< Creates a new recorder for the given Channel and
        ///< the given Priority that will record into the file FileName.
   virtual ~cRecorder();
+  int Errors(void) { return oldErrors + errors; };
+       ///< Returns the number of errors that were detected during recording.
   };
 
 #endif //__RECORDER_H
