@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.h 4.8 2018/02/12 12:28:24 kls Exp $
+ * $Id: recording.h 5.8 2024/06/13 09:31:11 kls Exp $
  */
 
 #ifndef __RECORDING_H
@@ -17,6 +17,7 @@
 #include "thread.h"
 #include "timers.h"
 #include "tools.h"
+#include "remux.h"
 
 #define FOLDERDELIMCHAR '~'
 
@@ -69,13 +70,16 @@ private:
   cEvent *ownEvent;
   char *aux;
   double framesPerSecond;
+  uint16_t frameWidth;
+  uint16_t frameHeight;
+  eScanType scanType;
+  eAspectRatio aspectRatio;
   int priority;
   int lifetime;
   char *fileName;
+  int errors;
   cRecordingInfo(const cChannel *Channel = NULL, const cEvent *Event = NULL);
   bool Read(FILE *f);
-  void SetData(const char *Title, const char *ShortText, const char *Description);
-  void SetAux(const char *Aux);
 public:
   cRecordingInfo(const char *FileName);
   ~cRecordingInfo();
@@ -88,11 +92,23 @@ public:
   const cComponents *Components(void) const { return event->Components(); }
   const char *Aux(void) const { return aux; }
   double FramesPerSecond(void) const { return framesPerSecond; }
+  uint16_t FrameWidth(void) const { return frameWidth; }
+  uint16_t FrameHeight(void) const { return frameHeight; }
+  eScanType ScanType(void) const { return scanType; }
+  char ScanTypeChar(void) const { return ScanTypeChars[scanType]; }
+  eAspectRatio AspectRatio(void) const { return aspectRatio; }
+  const char *AspectRatioText(void) const { return AspectRatioTexts[aspectRatio]; }
+  cString FrameParams(void) const;
   void SetFramesPerSecond(double FramesPerSecond);
+  void SetFrameParams(uint16_t FrameWidth, uint16_t FrameHeight, eScanType ScanType, eAspectRatio AspectRatio);
   void SetFileName(const char *FileName);
+  int Errors(void) const { return errors; } // returns -1 if undefined
+  void SetErrors(int Errors);
   bool Write(FILE *f, const char *Prefix = "") const;
   bool Read(void);
   bool Write(void) const;
+  void SetData(const char *Title, const char *ShortText, const char *Description);
+  void SetAux(const char *Aux);
   };
 
 class cRecording : public cListObject {
@@ -150,7 +166,7 @@ public:
        ///< Returns the full path name to the recording directory, including the
        ///< video directory and the actual '*.rec'. For disk file access use.
   const char *Title(char Delimiter = ' ', bool NewIndicator = false, int Level = -1) const;
-  const cRecordingInfo *Info(void) const { return info; }
+  cRecordingInfo *Info(void) const { return info; }
   const char *PrefixFileName(char Prefix);
   int HierarchyLevels(void) const;
   void ResetResume(void) const;
@@ -158,8 +174,15 @@ public:
   int NumFrames(void) const;
        ///< Returns the number of frames in this recording.
        ///< If the number of frames is unknown, -1 will be returned.
+  int NumFramesAfterEdit(void) const;
+       ///< Returns the number of frames in the edited version of this recording.
+       ///< If there are no editing marks, 0 will be returned.
+       ///< If the number of frames is unknown, -1 will be returned.
   int LengthInSeconds(void) const;
        ///< Returns the length (in seconds) of this recording, or -1 in case of error.
+  int LengthInSecondsAfterEdit(void) const;
+       ///< Returns the length (in seconds) of the edited version of this recording, or -1 in case of error.
+       ///< If there are no editing marks, 0 will be returned.
   int FileSizeMB(void) const;
        ///< Returns the total file size of this recording (in MB), or -1 if the file
        ///< size is unknown.
@@ -254,8 +277,6 @@ public:
        ///< Touches the '.update' file in the video directory, so that other
        ///< instances of VDR that access the same video directory can be triggered
        ///< to update their recordings list.
-       ///< This function is 'const', because it doesn't actually modify the list
-       ///< of recordings.
   static bool NeedsUpdate(void);
   void ResetResume(const char *ResumeFileName = NULL);
   void ClearSortNames(void);
@@ -339,6 +360,10 @@ public:
        ///< Deletes/terminates all operations.
   int GetUsage(const char *FileName);
        ///< Returns the usage type for the given FileName.
+  int GetRequiredDiskSpaceMB(const char *FileName = NULL);
+       ///< Returns the total disk space required to process all actions.
+       ///< If FileName is given, only the drive that contains that file is taken
+       ///< into account.
   bool Finished(bool &Error);
        ///< Returns true if all operations in the list have been finished.
        ///< If there have been any errors, Errors will be set to true.
@@ -392,7 +417,7 @@ public:
        ///< If this cMarks object is used by multiple threads, the caller must Lock()
        ///< it before calling Add() and Unlock() it afterwards. The same applies to
        ///< calls to Del(), or any of the functions that return a "cMark *", in case
-       ///< an other thread might modifiy the list while the returned pointer is
+       ///< an other thread might modify the list while the returned pointer is
        ///< considered valid.
   const cMark *Get(int Position) const;
   const cMark *GetPrev(int Position) const;
@@ -410,6 +435,10 @@ public:
        ///< 0 (the beginning of the recording), and there is no "end" mark, the
        ///< return value is 0, which means that the result is the same as the original
        ///< recording.
+  int GetFrameAfterEdit(int Frame, int LastFrame) const;
+       ///< Returns the number of the given Frame within the region covered by begin/end sequences.
+       ///< LastFrame must be given by the caller.
+       ///< If there are no editing marks or in case of an error -1 is returned.
   cMark *Get(int Position) { return const_cast<cMark *>(static_cast<const cMarks *>(this)->Get(Position)); }
   cMark *GetPrev(int Position) { return const_cast<cMark *>(static_cast<const cMarks *>(this)->GetPrev(Position)); }
   cMark *GetNext(int Position) { return const_cast<cMark *>(static_cast<const cMarks *>(this)->GetNext(Position)); }
@@ -423,6 +452,10 @@ public:
 #define RUC_EDITINGRECORDING "editing"
 #define RUC_EDITEDRECORDING  "edited"
 #define RUC_DELETERECORDING  "deleted"
+#define RUC_RENAMEDRECORDING "renamed" // same directory, only the base name is changed
+#define RUC_MOVEDRECORDING   "moved"   // different directory (and maybe base name), or "copy" to other filesystem + delete original (triggers copying->copied->deleted)
+#define RUC_COPYINGRECORDING "copying"
+#define RUC_COPIEDRECORDING  "copied"
 
 class cRecordingUserCommand {
 private:
@@ -506,6 +539,20 @@ public:
   cUnbufferedFile *NextFile(void);
   };
 
+class cDoneRecordings {
+private:
+  cString fileName;
+  cStringList doneRecordings;
+  void Add(const char *Title);
+public:
+  bool Load(const char *FileName);
+  bool Save(void) const;
+  void Append(const char *Title);
+  bool Contains(const char *Title) const;
+  };
+
+extern cDoneRecordings DoneRecordingsPattern;
+
 cString IndexToHMSF(int Index, bool WithFrame = false, double FramesPerSecond = DEFAULTFRAMESPERSECOND);
       // Converts the given index to a string, optionally containing the frame number.
 int HMSFToIndex(const char *HMSF, double FramesPerSecond = DEFAULTFRAMESPERSECOND);
@@ -538,4 +585,7 @@ void IncRecordingsSortMode(const char *Directory);
 void SetRecordingTimerId(const char *Directory, const char *TimerId);
 cString GetRecordingTimerId(const char *Directory);
 
+int FileSizeMBafterEdit(const char *FileName);
+bool EnoughFreeDiskSpaceForEdit(const char *FileName);
+
 #endif //__RECORDING_H
