@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: sdt.c 4.5 2015/08/02 11:33:23 kls Exp $
+ * $Id: sdt.c 5.2 2021/06/21 20:13:55 kls Exp $
  */
 
 #include "sdt.h"
@@ -23,8 +23,13 @@ static bool DebugSdt = false;
 cSdtFilter::cSdtFilter(cPatFilter *PatFilter)
 {
   source = cSource::stNone;
+  lastSource = cSource::stNone;
+  lastTransponder = 0;
+  lastNid = 0;
+  lastTid = 0;
   patFilter = PatFilter;
-  Set(0x11, 0x42);  // SDT
+  transponderState = tsUnknown;
+  Set(0x11, 0x42);  // SDT actual TS
 }
 
 void cSdtFilter::SetStatus(bool On)
@@ -34,6 +39,7 @@ void cSdtFilter::SetStatus(bool On)
   sectionSyncer.Reset();
   if (!On)
      source = cSource::stNone;
+  transponderState = tsUnknown;
 }
 
 void cSdtFilter::Trigger(int Source)
@@ -45,21 +51,46 @@ void cSdtFilter::Trigger(int Source)
 void cSdtFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
 {
   cMutexLock MutexLock(&mutex);
-  if (!(source && Transponder()))
-     return;
   SI::SDT sdt(Data, false);
   if (!sdt.CheckCRCAndParse())
      return;
-  if (!sectionSyncer.Sync(sdt.getVersionNumber(), sdt.getSectionNumber(), sdt.getLastSectionNumber()))
+  if (transponderState == tsUnknown) {
+     // The transponder can be verified with any section, no sync required:
+     int Nid = sdt.getOriginalNetworkId();
+     int Tid = sdt.getTransportStreamId();
+     if (Source() != lastSource || !ISTRANSPONDER(Transponder(), lastTransponder)) {
+        // We expect a change in NID/TID:
+        if (Nid && Tid && Nid == lastNid && Tid == lastTid) {
+           transponderState = tsWrong;
+           dsyslog("SDT: channel %d NID/TID (%d/%d) not found, got %d/%d", Channel()->Number(), Channel()->Nid(), Channel()->Tid(), Nid, Tid);
+           return;
+           }
+        }
+     // NID/TID is acceptable:
+     lastSource = Source();
+     lastTransponder = Transponder();
+     lastNid = Nid;
+     lastTid = Tid;
+     if (Nid == Channel()->Nid() && Tid == Channel()->Tid()) {
+        // NID/TID correspond with the channel data:
+        transponderState = tsVerified;
+        }
+     else {
+        // NID/TID differ from the channel data, but we accept it, since this *is* the data for this transponder:
+        transponderState = tsAccepted;
+        }
+     }
+  if (!(source && Transponder()))
+     return;
+  if (!sectionSyncer.Check(sdt.getVersionNumber(), sdt.getSectionNumber()))
      return;
   cStateKey StateKey;
   cChannels *Channels = cChannels::GetChannelsWrite(StateKey, 10);
-  if (!Channels) {
-     sectionSyncer.Repeat(); // let's not miss any section of the SDT
+  if (!Channels)
      return;
-     }
   dbgsdt("SDT: %2d %2d %2d %s %d\n", sdt.getVersionNumber(), sdt.getSectionNumber(), sdt.getLastSectionNumber(), *cSource::ToString(source), Transponder());
   bool ChannelsModified = false;
+  bool TriggerPat = false;
   SI::SDT::Service SiSdtService;
   for (SI::Loop::Iterator it; sdt.serviceLoop.getNext(SiSdtService, it); ) {
       cChannel *Channel = Channels->GetByChannelID(tChannelID(source, sdt.getOriginalNetworkId(), sdt.getTransportStreamId(), SiSdtService.getServiceId()));
@@ -79,8 +110,11 @@ void cSdtFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                    case 0x02: // digital radio sound service
                    case 0x04: // NVOD reference service
                    case 0x05: // NVOD time-shifted service
+                   case 0x0A: // advanced codec digital radio sound service
                    case 0x16: // digital SD television service
                    case 0x19: // digital HD television service
+                   case 0x1F: // HEVC digital television service
+                   case 0x20: // HEVC UHD digital television service
                         {
                         char NameBuf[Utf8BufSize(1024)];
                         char ShortNameBuf[Utf8BufSize(1024)];
@@ -119,7 +153,7 @@ void cSdtFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                            Channel = Channels->NewChannel(this->Channel(), pn, ps, pp, sdt.getOriginalNetworkId(), sdt.getTransportStreamId(), SiSdtService.getServiceId());
                            Channel->SetSource(source); // in case this comes from a satellite with a slightly different position
                            ChannelsModified = true;
-                           patFilter->Trigger(SiSdtService.getServiceId());
+                           TriggerPat = true;
                            }
                         }
                    default: ;
@@ -145,8 +179,8 @@ void cSdtFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                      cChannel *link = Channels->GetByChannelID(tChannelID(source, Service.getOriginalNetworkId(), Service.getTransportStream(), Service.getServiceId()));
                      if (!link && Setup.UpdateChannels >= 4) {
                         link = Channels->NewChannel(this->Channel(), "NVOD", "", "", Service.getOriginalNetworkId(), Service.getTransportStream(), Service.getServiceId());
-                        patFilter->Trigger(Service.getServiceId());
                         ChannelsModified = true;
+                        TriggerPat = true;
                         }
                      if (link) {
                         if (!LinkChannels)
@@ -168,7 +202,9 @@ void cSdtFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
             delete LinkChannels;
          }
       }
-  if (sdt.getSectionNumber() == sdt.getLastSectionNumber()) {
+  if (TriggerPat)
+     patFilter->Trigger();
+  if (sectionSyncer.Processed(sdt.getSectionNumber(), sdt.getLastSectionNumber())) {
      if (Setup.UpdateChannels == 1 || Setup.UpdateChannels >= 3) {
         ChannelsModified |= Channels->MarkObsoleteChannels(source, sdt.getOriginalNetworkId(), sdt.getTransportStreamId());
         if (source != Source())
