@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: sections.h 4.0 2005/08/13 11:23:55 kls Exp $
+ * $Id: sections.h 5.1 2021/06/08 15:10:51 kls Exp $
  */
 
 #ifndef __SECTIONS_H
@@ -27,7 +27,9 @@ private:
   cDevice *device;
   int statusCount;
   bool on, waitForLock;
-  time_t lastIncompleteSection;
+  bool flush;
+  bool startFilters;
+  cTimeMs flushTimer;
   cList<cFilter> filters;
   cList<cFilterHandle> filterHandles;
   void Add(const cFilterData *FilterData);
