@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skinclassic.c 4.1 2016/12/22 14:07:04 kls Exp $
+ * $Id: skinclassic.c 5.2 2023/12/29 10:48:40 kls Exp $
  */
 
 #include "skinclassic.h"
@@ -393,11 +393,20 @@ void cSkinClassicDisplayMenu::SetRecording(const cRecording *Recording)
   cString t = cString::sprintf("%s  %s  %s", *DateString(Recording->Start()), *TimeString(Recording->Start()), Info->ChannelName() ? Info->ChannelName() : "");
   ts.Set(osd, x1, y, x2 - x1, y3 - y, t, font, Theme.Color(clrMenuEventTime), Theme.Color(clrBackground));
   y += ts.Height();
+  int xt = x3;
   if (Info->GetEvent()->ParentalRating()) {
      cString buffer = cString::sprintf(" %s ", *Info->GetEvent()->GetParentalRatingString());
      const cFont *font = cFont::GetFont(fontSml);
      int w = font->Width(buffer);
-     osd->DrawText(x3 - w, y, buffer, Theme.Color(clrMenuEventVpsFg), Theme.Color(clrMenuEventVpsBg), font, w);
+     osd->DrawText(xt - w, y, buffer, Theme.Color(clrMenuEventVpsFg), Theme.Color(clrMenuEventVpsBg), font, w);
+     xt -= w + 5;
+     }
+  if (Info->Errors() > 0) {
+     cString buffer = cString::sprintf(" %d %s ", Info->Errors(), tr("errors"));
+     const cFont *font = cFont::GetFont(fontSml);
+     int w = font->Width(buffer);
+     osd->DrawText(xt - w, y, buffer, Theme.Color(clrMenuEventVpsFg), Theme.Color(clrMenuEventVpsBg), font, w);
+     xt -= w + 5;
      }
   y += font->Height();
   const char *Title = Info->Title();
@@ -412,7 +421,13 @@ void cSkinClassicDisplayMenu::SetRecording(const cRecording *Recording)
      }
   y += font->Height();
   if (!isempty(Info->Description())) {
-     textScroller.Set(osd, x1, y, x2 - x1, y3 - y, Info->Description(), font, Theme.Color(clrMenuEventDescription), Theme.Color(clrBackground));
+     cString d = Info->Description();
+     cString f = Info->FrameParams();
+     if (*f) {
+        d.Append("\n\n");
+        d.Append(f);
+        }
+     textScroller.Set(osd, x1, y, x2 - x1, y3 - y, d, font, Theme.Color(clrMenuEventDescription), Theme.Color(clrBackground));
      SetTextScrollbar();
      }
 }
