@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 5.3 2021/01/14 10:29:05 kls Exp $
+ * $Id: svdrp.c 5.8 2024/06/13 09:31:11 kls Exp $
  */
 
 #include "svdrp.h"
@@ -521,7 +521,6 @@ bool cSVDRPClient::GetRemoteTimers(cStringList &Response)
   return false;
 }
 
-
 // --- cSVDRPServerParams ----------------------------------------------------
 
 class cSVDRPServerParams {
@@ -605,8 +604,6 @@ protected:
 public:
   cSVDRPClientHandler(int TcpPort, int UdpPort);
   virtual ~cSVDRPClientHandler();
-  void Lock(void) { mutex.Lock(); }
-  void Unlock(void) { mutex.Unlock(); }
   void AddClient(cSVDRPServerParams &ServerParams, const char *IpAddress);
   bool Execute(const char *ServerName, const char *Command, cStringList *Response = NULL);
   bool GetServerNames(cStringList *ServerNames);
@@ -1508,7 +1505,9 @@ void cSVDRPServer::CmdEDIT(const char *Option)
         if (const cRecording *Recording = Recordings->GetById(strtol(Option, NULL, 10))) {
            cMarks Marks;
            if (Marks.Load(Recording->FileName(), Recording->FramesPerSecond(), Recording->IsPesRecording()) && Marks.Count()) {
-              if (RecordingsHandler.Add(ruCut, Recording->FileName()))
+              if (!EnoughFreeDiskSpaceForEdit(Recording->FileName()))
+                 Reply(550, "Not enough free disk space to start editing process");
+              else if (RecordingsHandler.Add(ruCut, Recording->FileName()))
                  Reply(250, "Editing recording \"%s\" [%s]", Option, Recording->Title());
               else
                  Reply(554, "Can't start editing process");
@@ -2232,9 +2231,9 @@ void cSVDRPServer::CmdNEXT(const char *Option)
      if (!*Option)
         Reply(250, "%d %s", Id, *TimeToString(Start));
      else if (strcasecmp(Option, "ABS") == 0)
-        Reply(250, "%d %ld", Id, Start);
+        Reply(250, "%d %jd", Id, intmax_t(Start));
      else if (strcasecmp(Option, "REL") == 0)
-        Reply(250, "%d %ld", Id, Start - time(NULL));
+        Reply(250, "%d %jd", Id, intmax_t(Start - time(NULL)));
      else
         Reply(501, "Unknown option: \"%s\"", Option);
      }
@@ -2382,10 +2381,8 @@ void cSVDRPServer::CmdPOLL(const char *Option)
      if (SVDRPClientHandler) {
         if (ListName) {
            if (strcasecmp(ListName, "timers") == 0) {
-              if (SVDRPClientHandler->TriggerFetchingTimers(RemoteName))
-                 Reply(250, "OK");
-              else
-                 Reply(501, "No connection to \"%s\"", RemoteName);
+              Reply(250, "OK"); // must send reply before calling TriggerFetchingTimers() to avoid a deadlock if two clients send each other POLL commands at the same time
+              SVDRPClientHandler->TriggerFetchingTimers(RemoteName);
               }
            else
               Reply(501, "Unknown list name: \"%s\"", ListName);
@@ -2796,11 +2793,8 @@ bool GetSVDRPServerNames(cStringList *ServerNames)
 {
   bool Result = false;
   cMutexLock MutexLock(&SVDRPHandlerMutex);
-  if (SVDRPClientHandler) {
-     SVDRPClientHandler->Lock();
+  if (SVDRPClientHandler)
      Result = SVDRPClientHandler->GetServerNames(ServerNames);
-     SVDRPClientHandler->Unlock();
-     }
   return Result;
 }
 
@@ -2808,11 +2802,8 @@ bool ExecSVDRPCommand(const char *ServerName, const char *Command, cStringList *
 {
   bool Result = false;
   cMutexLock MutexLock(&SVDRPHandlerMutex);
-  if (SVDRPClientHandler) {
-     SVDRPClientHandler->Lock();
+  if (SVDRPClientHandler)
      Result = SVDRPClientHandler->Execute(ServerName, Command, Response);
-     SVDRPClientHandler->Unlock();
-     }
   return Result;
 }
 
@@ -2821,11 +2812,9 @@ void BroadcastSVDRPCommand(const char *Command)
   cMutexLock MutexLock(&SVDRPHandlerMutex);
   cStringList ServerNames;
   if (SVDRPClientHandler) {
-     SVDRPClientHandler->Lock();
      if (SVDRPClientHandler->GetServerNames(&ServerNames)) {
         for (int i = 0; i < ServerNames.Size(); i++)
             ExecSVDRPCommand(ServerNames[i], Command);
         }
-     SVDRPClientHandler->Unlock();
      }
 }
