@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 5.17 2021/05/03 14:06:12 kls Exp $
+ * $Id: timers.c 5.20 2024/03/06 14:37:15 kls Exp $
  */
 
 #include "timers.h"
@@ -21,6 +21,8 @@
 // format characters in order to allow any number of blanks after a numeric
 // value!
 
+#define VPSGRACE 15  // seconds we still record after the running status of a VPS event has changed to "not running"
+
 // --- cTimer ----------------------------------------------------------------
 
 cTimer::cTimer(bool Instant, bool Pause, const cChannel *Channel)
@@ -47,6 +49,8 @@ cTimer::cTimer(bool Instant, bool Pause, const cChannel *Channel)
   weekdays = 0;
   start = now->tm_hour * 100 + now->tm_min;
   stop = 0;
+  vpsNotRunning = 0;
+  vpsActive = false;
   if (!Setup.InstantRecordTime && channel && (Instant || Pause)) {
      LOCK_SCHEDULES_READ;
      if (const cSchedule *Schedule = Schedules->GetSchedule(channel)) {
@@ -186,6 +190,8 @@ cTimer::cTimer(const cEvent *Event, const char *FileName, const cTimer *PatternT
   aux = NULL;
   remote = NULL;
   event = NULL;
+  vpsNotRunning = 0;
+  vpsActive = false;
   if (!PatternTimer || PatternTimer->HasFlags(tfVps)) {
      if (Event->Vps() && (PatternTimer || Setup.UseVps))
         SetFlags(tfVps);
@@ -255,6 +261,8 @@ cTimer& cTimer::operator= (const cTimer &Timer)
      stop         = Timer.stop;
      priority     = Timer.priority;
      lifetime     = Timer.lifetime;
+     vpsNotRunning = 0;
+     vpsActive    = false;
      strncpy(pattern, Timer.pattern, sizeof(pattern));
      strncpy(file, Timer.file, sizeof(file));
      free(aux);
@@ -556,6 +564,7 @@ void cTimer::SetFile(const char *File)
 }
 
 #define EITPRESENTFOLLOWINGRATE 10 // max. seconds between two occurrences of the "EIT present/following table for the actual multiplex" (2s by the standard, using some more for safety)
+#define EITPRESENTFOLLOWINGGRACE 60 // max. seconds before reporting a loss of VPS control of an ongoing recording
 
 bool cTimer::Matches(time_t t, bool Directly, int Margin) const
 {
@@ -611,9 +620,32 @@ bool cTimer::Matches(time_t t, bool Directly, int Margin) const
                  startTime = event->StartTime();
                  stopTime = event->EndTime();
                  if (!Margin) { // this is an actual check
-                    if (event->Schedule()->PresentSeenWithin(EITPRESENTFOLLOWINGRATE)) // VPS control can only work with up-to-date events...
-                       return event->IsRunning(true);
-                    // ...otherwise we fall back to normal timer handling below (note: Margin == 0!)
+                    if (event->Schedule()->PresentSeenWithin(EITPRESENTFOLLOWINGRATE)) { // VPS control can only work with up-to-date events...
+                       if (!vpsActive) {
+                          vpsActive = true;
+                          if (Recording())
+                             dsyslog("timer %s regained VPS control", *ToDescr());
+                          }
+                       bool running = event->IsRunning(true);
+                       if (!running) {
+                          if (Recording() && vpsNotRunning == 0)
+                             vpsNotRunning = time(NULL);
+                          }
+                       else
+                          vpsNotRunning = 0;
+                       return running || time(NULL) < vpsNotRunning + VPSGRACE;
+                       }
+                    if (Recording()) {
+                       if (event->Schedule()->PresentSeenWithin(EITPRESENTFOLLOWINGGRACE))
+                          return event->IsRunning(true); // give it a chance to recover - worst case: the recording will be 60 seconds too long
+                       if (vpsActive) {
+                          vpsActive = false;
+                          esyslog("ERROR: timer %s lost VPS control", *ToDescr());
+                          }
+                       // ...otherwise we fall back to normal timer handling below (note: Margin == 0!)
+                       }
+                    else
+                       return false; // relying on vdr.c to ensure that a transponder is tuned to this channel
                     }
                  }
               }
@@ -1131,7 +1163,7 @@ const cTimer *cTimers::GetMatch(const cEvent *Event, eTimerMatch *Match) const
   return t;
 }
 
-const cTimer *cTimers::GetTimerForEvent(const cEvent *Event, eTimerFlags Flags)
+const cTimer *cTimers::GetTimerForEvent(const cEvent *Event, eTimerFlags Flags) const
 {
   if (Event && Event->HasTimer()) {
      for (const cTimer *ti = First(); ti; ti = Next(ti)) {
