@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 5.4 2021/05/20 10:13:43 kls Exp $
+ * $Id: tools.c 5.12 2024/07/10 14:50:07 kls Exp $
  */
 
 #include "tools.h"
@@ -151,26 +151,31 @@ char *strreplace(char *s, char c1, char c2)
 
 char *strreplace(char *s, const char *s1, const char *s2)
 {
-  if (!s || !s1 || !s2)
+  if (!s || !s1 || !s2 || strcmp(s1, s2) == 0)
      return s;
-  char *p = strstr(s, s1);
-  if (p) {
-     int of = p - s;
+  char *q = s;
+  if (char *p = strstr(s, s1)) {
      int l  = strlen(s);
      int l1 = strlen(s1);
      int l2 = strlen(s2);
-     if (l2 > l1) {
-        if (char *NewBuffer = (char *)realloc(s, l + l2 - l1 + 1))
-           s = NewBuffer;
-        else {
-           esyslog("ERROR: out of memory");
-           return s;
+     do {
+        int of = p - s;
+        if (l2 > l1) {
+           if (char *NewBuffer = (char *)realloc(s, l + l2 - l1 + 1))
+              s = NewBuffer;
+           else {
+              esyslog("ERROR: out of memory");
+              return s;
+              }
            }
-        }
-     char *sof = s + of;
-     if (l2 != l1)
-        memmove(sof + l2, sof + l1, l - of - l1 + 1);
-     strncpy(sof, s2, l2);
+        char *sof = s + of;
+        if (l2 != l1) {
+           memmove(sof + l2, sof + l1, l - of - l1 + 1);
+           l += l2 - l1;
+           }
+        memcpy(sof, s2, l2);
+        q = sof + l2;
+        } while (p = strstr(q, s1));
      }
   return s;
 }
@@ -767,7 +772,7 @@ uint64_t cTimeMs::Now(void)
               esyslog("cTimeMs: clock_gettime(CLOCK_MONOTONIC) failed");
            }
         else
-           dsyslog("cTimeMs: not using monotonic clock - resolution is too bad (%ld s %ld ns)", tp.tv_sec, tp.tv_nsec);
+           dsyslog("cTimeMs: not using monotonic clock - resolution is too bad (%jd s %ld ns)", intmax_t(tp.tv_sec), tp.tv_nsec);
         }
      else
         esyslog("cTimeMs: clock_getres(CLOCK_MONOTONIC) failed");
@@ -1106,6 +1111,16 @@ cString &cString::operator=(const cString &String)
   return *this;
 }
 
+cString &cString::operator=(cString &&String)
+{
+  if (this != &String) {
+     free(s);
+     s = String.s;
+     String.s = NULL;
+     }
+  return *this;
+}
+
 cString &cString::operator=(const char *String)
 {
   if (s == String)
@@ -1130,6 +1145,22 @@ cString &cString::Append(const char *String)
   return *this;
 }
 
+cString &cString::Append(char c)
+{
+  if (c) {
+     int l1 = s ? strlen(s) : 0;
+     int l2 = 1;
+     if (char *p = (char *)realloc(s, l1 + l2 + 1)) {
+        s = p;
+        *(s + l1) = c;
+        *(s + l1 + 1) = 0;
+        }
+     else
+        esyslog("ERROR: out of memory");
+     }
+  return *this;
+}
+
 cString &cString::Truncate(int Index)
 {
   int l = strlen(s);
@@ -2050,6 +2081,10 @@ bool cLockFile::Lock(int WaitSeconds)
               }
            else {
               LOG_ERROR_STR(fileName);
+              if (errno == ENOSPC) {
+                 esyslog("ERROR: can't create lock file '%s' - assuming lock anyway!", fileName);
+                 return true;
+                 }
               break;
               }
            if (WaitSeconds)
