@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.h 4.16 2018/03/04 14:06:36 kls Exp $
+ * $Id: tools.h 5.8 2024/01/20 13:59:55 kls Exp $
  */
 
 #ifndef __TOOLS_H
@@ -51,19 +51,21 @@ template<class T> inline void DELETENULL(T *&p) { T *q = p; p = NULL; delete q;
 #define CHECK(s) { if ((s) < 0) LOG_ERROR; } // used for 'ioctl()' calls
 #define FATALERRNO (errno && errno != EAGAIN && errno != EINTR)
 
-// In case some plugin needs to use the STL and gets an error message regarding one
-// of these functions, you can #define DISABLE_TEMPLATES_COLLIDING_WITH_STL before
-// including tools.h.
-#if !defined(__STL_CONFIG_H) // for old versions of the STL
-#if !defined(DISABLE_TEMPLATES_COLLIDING_WITH_STL) && !defined(_STL_ALGOBASE_H)
+#if __cplusplus >= 201103L
+// any gcc >= 4.8.1; we have swap, min, max
+#include <algorithm> // std::min, std::max, (c++98: also swap)
+#include <utility> // std::swap (since c++11)
+using std::min;
+using std::max;
+using std::swap;
+#else
+// no c++11 and old compiler, let's include our own templates
 template<class T> inline T min(T a, T b) { return a <= b ? a : b; }
 template<class T> inline T max(T a, T b) { return a >= b ? a : b; }
-#endif
-template<class T> inline int sgn(T a) { return a < 0 ? -1 : a > 0 ? 1 : 0; }
-#if !defined(DISABLE_TEMPLATES_COLLIDING_WITH_STL) && !defined(_MOVE_H)
 template<class T> inline void swap(T &a, T &b) { T t = a; a = b; b = t; }
 #endif
-#endif
+
+template<class T> inline int sgn(T a) { return a < 0 ? -1 : a > 0 ? 1 : 0; }
 
 template<class T> inline T constrain(T v, T l, T h) { return v < l ? l : v > h ? h : v; }
 
@@ -180,19 +182,49 @@ public:
   cString(const char *S = NULL, bool TakePointer = false);
   cString(const char *S, const char *To); ///< Copies S up to To (exclusive). To must be a valid pointer into S. If To is NULL, everything is copied.
   cString(const cString &String);
+  cString(cString &&String): s(String.s) { String.s = NULL; }
   virtual ~cString();
   operator const void * () const { return s; } // to catch cases where operator*() should be used
   operator const char * () const { return s; } // for use in (const char *) context
   const char * operator*() const { return s; } // for use in (const void *) context (printf() etc.)
   cString &operator=(const cString &String);
+  cString &operator=(cString &&String);
   cString &operator=(const char *String);
   cString &Append(const char *String);
+  cString &Append(char c);
   cString &Truncate(int Index); ///< Truncate the string at the given Index (if Index is < 0 it is counted from the end of the string).
   cString &CompactChars(char c); ///< Compact any sequence of characters 'c' to a single character, and strip all of them from the beginning and end of this string.
   static cString sprintf(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
   static cString vsprintf(const char *fmt, va_list &ap);
   };
 
+class cNullTerminate {
+private:
+  char *p;
+  char c;
+public:
+  cNullTerminate(void) {
+    p = NULL;
+    c = 0;
+    }
+  cNullTerminate(char *s) {
+    Set(s);
+    }
+  ~cNullTerminate() {
+    if (p)
+       *p = c;
+    }
+  void Set(char *s) {
+    if (s) {
+       p = s;
+       c = *s;
+       *s = 0;
+       }
+    else
+       p = NULL;
+    }
+  };
+
 ssize_t safe_read(int filedes, void *buffer, size_t size);
 ssize_t safe_write(int filedes, const void *buffer, size_t size);
 void writechar(int filedes, char c);
@@ -206,6 +238,9 @@ char *strreplace(char *s, char c1, char c2);
 char *strreplace(char *s, const char *s1, const char *s2); ///< re-allocates 's' and deletes the original string if necessary!
 const char *strchrn(const char *s, char c, size_t n); ///< returns a pointer to the n'th occurrence (counting from 1) of c in s, or NULL if no such character was found. If n is 0, s is returned.
 int strcountchr(const char *s, char c); ///< returns the number of occurrences of 'c' in 's'.
+cString strgetbefore(const char *s, char c, int n = 1); // returns the part of 's' before (and excluding) the n'th occurrence of 'c' from the right, or an empty string if there is no such 'c'.
+const char *strgetlast(const char *s, char c); // returns the part of 's' after the last occurrence of 'c', or 's' if there is no 'c'.
+inline char *strgetlast(char *s, char c) { return const_cast<char *>(strgetlast(static_cast<const char *>(s), c)); } // returns the part of 's' after the last occurrence of 'c', or 's' if there is no 'c'.
 inline char *skipspace(const char *s)
 {
   if ((uchar)*s > ' ') // most strings don't have any leading space, so handle this case as fast as possible
@@ -222,7 +257,7 @@ cString strgetval(const char *s, const char *name, char d = '=');
     ///< Returns the value part of a 'name=value' pair in s.
     ///< name must either be at the beginning of s, or has to be preceded by white space.
     ///< There may be any number of white space around the '=' sign. The value is
-    ///< everyting up to (and excluding) the next white space, or the end of s.
+    ///< everything up to (and excluding) the next white space, or the end of s.
     ///< If an other delimiter shall be used (like, e.g., ':'), it can be given
     ///< as the third parameter.
     ///< If name occurs more than once in s, only the first occurrence is taken.
@@ -376,6 +411,12 @@ public:
       ///< time.
   static uint64_t Now(void);
   void Set(int Ms = 0);
+      ///< Sets the timer. If Ms is 0, call Elapsed() to get the number of milliseconds
+      ///< since the timer has been set. If Ms is greater than 0, TimedOut() returns
+      ///< true as soon as Ms milliseconds have passed since calling Set(). If Ms is
+      ///< negative, results are undefined.
+      ///< Depending on the value of Ms, an object of cTimeMs can handle either
+      ///< timeouts or elapsed times, not both at the same time.
   bool TimedOut(void) const;
   uint64_t Elapsed(void) const;
   };
@@ -419,10 +460,15 @@ public:
   struct dirent *Next(void);
   };
 
+#ifndef DEPRECATED_CFILE
+#define DEPRECATED_CFILE 0
+#endif
 class cFile {
 private:
+#if DEPRECATED_CFILE
   static bool files[];
   static int maxFiles;
+#endif
   int f;
 public:
   cFile(void);
@@ -433,9 +479,13 @@ public:
   void Close(void);
   bool IsOpen(void) { return f >= 0; }
   bool Ready(bool Wait = true);
+#if DEPRECATED_CFILE
   static bool AnyFileReady(int FileDes = -1, int TimeoutMs = 1000);
+#endif
   static bool FileReady(int FileDes, int TimeoutMs = 1000);
+#if DEPRECATED_CFILE
   static bool FileReadyForWriting(int FileDes, int TimeoutMs = 1000);
+#endif
   };
 
 class cSafeFile {
