@@ -22,13 +22,14 @@
  *
  * The project's page is at http://www.tvdr.de
  *
- * $Id: vdr.c 5.7 2021/12/27 13:31:04 kls Exp $
+ * $Id: vdr.c 5.16 2024/03/29 21:46:50 kls Exp $
  */
 
 #include <getopt.h>
 #include <grp.h>
 #include <langinfo.h>
 #include <locale.h>
+#include <malloc.h>
 #include <pwd.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -74,6 +75,7 @@
 
 #define MINCHANNELWAIT        10 // seconds to wait between failed channel switchings
 #define ACTIVITYTIMEOUT       60 // seconds before starting housekeeping
+#define MEMCLEANUPDELTA     3600 // seconds between memory cleanups
 #define SHUTDOWNWAIT         300 // seconds to wait in user prompt before automatic shutdown
 #define SHUTDOWNRETRY        360 // seconds before trying again to shut down
 #define SHUTDOWNFORCEPROMPT    5 // seconds to wait in user prompt to allow forcing shutdown
@@ -83,7 +85,7 @@
 #define CHANNELSAVEDELTA     600 // seconds before saving channels.conf after automatic modifications
 #define DEVICEREADYTIMEOUT    30 // seconds to wait until all devices are ready
 #define MENUTIMEOUT          120 // seconds of user inactivity after which an OSD display is closed
-#define TIMERCHECKDELTA       10 // seconds between checks for timers that need to see their channel
+#define TIMERCHECKDELTA        5 // seconds between checks for timers that need to see their channel
 #define TIMERDEVICETIMEOUT     8 // seconds before a device used for timer check may be reused
 #define TIMERLOOKAHEADTIME    60 // seconds before a non-VPS timer starts and the channel is switched if possible
 #define VPSLOOKAHEADTIME      24 // hours within which VPS timers will make sure their events are up to date
@@ -525,22 +527,6 @@ int main(int argc, char *argv[])
           }
         }
 
-  // Set user id in case we were started as root:
-
-  if (VdrUser && geteuid() == 0) {
-     StartedAsRoot = true;
-     if (strcmp(VdrUser, "root") && strcmp(VdrUser, "0")) {
-        if (!SetKeepCaps(true))
-           return 2;
-        if (!SetUser(VdrUser, UserDump))
-           return 2;
-        if (!SetKeepCaps(false))
-           return 2;
-        if (!DropCaps())
-           return 2;
-        }
-     }
-
   // Help and version info:
 
   if (DisplayHelp || DisplayVersion) {
@@ -665,13 +651,6 @@ int main(int argc, char *argv[])
   if (SysLogLevel > 0)
      openlog("vdr", LOG_CONS, SysLogTarget); // LOG_PID doesn't work as expected under NPTL
 
-  // Check the video directory:
-
-  if (!DirectoryOk(VideoDirectory, true)) {
-     fprintf(stderr, "vdr: can't access video directory %s\n", VideoDirectory);
-     return 2;
-     }
-
   // Daemon mode:
 
   if (DaemonMode) {
@@ -690,6 +669,29 @@ int main(int argc, char *argv[])
      tcgetattr(STDIN_FILENO, &savedTm);
      }
 
+  // Set user id in case we were started as root:
+
+  if (VdrUser && geteuid() == 0) {
+     StartedAsRoot = true;
+     if (strcmp(VdrUser, "root") && strcmp(VdrUser, "0")) {
+        if (!SetKeepCaps(true))
+           return 2;
+        if (!SetUser(VdrUser, UserDump))
+           return 2;
+        if (!SetKeepCaps(false))
+           return 2;
+        if (!DropCaps())
+           return 2;
+        }
+     }
+
+  // Check the video directory:
+
+  if (!DirectoryOk(VideoDirectory, true)) {
+     fprintf(stderr, "vdr: can't access video directory %s\n", VideoDirectory);
+     return 2;
+     }
+
   isyslog("VDR version %s started", VDRVERSION);
   if (StartedAsRoot && VdrUser)
      isyslog("switched to user '%s'", VdrUser);
@@ -735,7 +737,6 @@ int main(int argc, char *argv[])
   time_t LastChannelChanged = time(NULL);
   time_t LastInteract = 0;
   int MaxLatencyTime = 0;
-  bool InhibitEpgScan = false;
   bool IsInfoMenu = false;
   cSkin *CurrentSkin = NULL;
   int OldPrimaryDVB = 0;
@@ -875,7 +876,7 @@ int main(int argc, char *argv[])
 
   // Remote Controls:
   if (LircDevice)
-     new cLircRemote(LircDevice);
+     cLircRemote::NewLircRemote(LircDevice);
   if (!DaemonMode && HasStdin && UseKbd)
      new cKbdRemote;
   Interface->LearnKeys();
@@ -915,10 +916,12 @@ int main(int argc, char *argv[])
     LOCK_CHANNELS_READ;
     Channels->SwitchTo(Setup.CurrentChannel);
   }
+
+  // Restore volume:
+
+  cDevice::PrimaryDevice()->SetVolume(Setup.CurrentVolume, true);
   if (MuteAudio)
      cDevice::PrimaryDevice()->ToggleMute();
-  else
-     cDevice::PrimaryDevice()->SetVolume(Setup.CurrentVolume, true);
 
   // Signal handlers:
 
@@ -1127,7 +1130,6 @@ int main(int argc, char *argv[])
           // Make sure timers "see" their channel early enough:
           static time_t LastTimerCheck = 0;
           if (Now - LastTimerCheck > TIMERCHECKDELTA) { // don't do this too often
-             InhibitEpgScan = false;
              for (cTimer *Timer = Timers->First(); Timer; Timer = Timers->Next(Timer)) {
                  if (Timer->Remote() || Timer->IsPatternTimer())
                     continue;
@@ -1135,10 +1137,8 @@ int main(int argc, char *argv[])
                  bool NeedsTransponder = false;
                  if (Timer->HasFlags(tfActive) && !Timer->Recording()) {
                     if (Timer->HasFlags(tfVps)) {
-                       if (Timer->Matches(Now, true, Setup.VpsMargin)) {
+                       if (Timer->Matches(Now, true, Setup.VpsMargin))
                           InVpsMargin = true;
-                          Timer->SetInVpsMargin(InVpsMargin);
-                          }
                        else if (Timer->Event()) {
                           InVpsMargin = Timer->Event()->StartTime() <= Now && Now < Timer->Event()->EndTime();
                           NeedsTransponder = Timer->Event()->StartTime() - Now < VPSLOOKAHEADTIME * 3600 && !Timer->Event()->SeenWithin(VPSUPTODATETIME);
@@ -1149,16 +1149,21 @@ int main(int argc, char *argv[])
                           InVpsMargin = !Schedule; // we must make sure we have the schedule
                           NeedsTransponder = Schedule && !Schedule->PresentSeenWithin(VPSUPTODATETIME);
                           }
-                       InhibitEpgScan |= InVpsMargin | NeedsTransponder;
                        }
                     else
                        NeedsTransponder = Timer->Matches(Now, true, TIMERLOOKAHEADTIME);
                     }
+                 if (!Timer->Recording())
+                    Timer->SetInVpsMargin(InVpsMargin);
                  if (NeedsTransponder || InVpsMargin) {
                     // Find a device that provides the required transponder:
                     cDevice *Device = cDevice::GetDeviceForTransponder(Timer->Channel(), MINPRIORITY);
-                    if (!Device && InVpsMargin)
-                       Device = cDevice::GetDeviceForTransponder(Timer->Channel(), LIVEPRIORITY);
+                    if (InVpsMargin) {
+                       if (!Device)
+                          Device = cDevice::GetDeviceForTransponder(Timer->Channel(), Timer->Priority() );
+                       if (!Device)
+                          Device = cDevice::GetDevice(Timer->Channel(), Timer->Priority(), false, false);
+                       }
                     // Switch the device to the transponder:
                     if (Device) {
                        bool HadProgramme = cDevice::PrimaryDevice()->HasProgramme();
@@ -1166,11 +1171,13 @@ int main(int argc, char *argv[])
                           if (Device == cDevice::ActualDevice() && !Device->IsPrimaryDevice())
                              cDevice::PrimaryDevice()->StopReplay(); // stop transfer mode
                           dsyslog("switching device %d to channel %d %s (%s)", Device->DeviceNumber() + 1, Timer->Channel()->Number(), *Timer->Channel()->GetChannelID().ToString(), Timer->Channel()->Name());
-                          if (Device->SwitchChannel(Timer->Channel(), false))
-                             Device->SetOccupied(TIMERDEVICETIMEOUT);
+                          Device->SwitchChannel(Timer->Channel(), false);
                           }
-                       if (cDevice::PrimaryDevice()->HasDecoder() && HadProgramme && !cDevice::PrimaryDevice()->HasProgramme())
+                       Device->SetOccupied(TIMERDEVICETIMEOUT, InVpsMargin ? Timer->Priority() : MINPRIORITY, Now);
+                       if (cDevice::PrimaryDevice()->HasDecoder() && HadProgramme && !cDevice::PrimaryDevice()->HasProgramme()) {
+                          LastTimerChannel = Timer->Channel()->Number();
                           Skins.QueueMessage(mtInfo, tr("Upcoming recording!")); // the previous SwitchChannel() has switched away the current live channel
+                          }
                        }
                     }
                  }
@@ -1515,8 +1522,7 @@ int main(int argc, char *argv[])
              }
            }
         if (!Menu) {
-           if (!InhibitEpgScan)
-              EITScanner.Process();
+           EITScanner.Process();
            bool Error = false;
            if (RecordingsHandler.Finished(Error)) {
               if (Error)
@@ -1578,6 +1584,12 @@ int main(int argc, char *argv[])
               cSchedules::Cleanup();
               // Plugins housekeeping:
               PluginManager.Housekeeping();
+              // Memory cleanup:
+              static time_t LastMemoryCleanup = 0;
+              if ((Now - LastMemoryCleanup) > MEMCLEANUPDELTA) {
+                 malloc_trim(0);
+                 LastMemoryCleanup = Now;
+                 }
               }
            }
 
